#
# Copyright (C) 2020 FreeIPA Contributors see COPYING for license
#

from ipalib import api
from ipahealthcheck.core import constants
from ipahealthcheck.core.plugin import Plugin, Result, Registry, duration
from ipaserver.install import dsinstance, installutils
try:
    from ipapython.ipaldap import realm_to_serverid
except ImportError:
    from ipaserver.install.installutils import realm_to_serverid
from lib389.cli_base import disconnect_instance, connect_instance
from lib389.properties import SER_LDAP_URL, SER_ROOT_DN


class DSArgs(dict):
    """
    Prepare the args to make a dirsrv connection that is compatible with
    lib389's Dirsrv object.
    """
    def __init__(self, inst):
        self.pwdfile = None
        self.bindpw = None
        self.prompt = False
        self.instance = inst


class DSPlugin(Plugin):
    requires = ('dirsrv',)
    check_class = None
    many = False

    def __init__(self, registry):
        super().__init__(registry)
        self.ds = self.ds = dsinstance.DsInstance()
        self.conn = api.Backend.ldap2
        self.serverid = realm_to_serverid(api.env.realm)

    def convertSev(self, ds_severity):
        """Convert lib389 HC severity level to IDM's HC level"""
        sev = ds_severity.lower()
        if sev == 'high':
            return constants.CRITICAL
        if sev == 'medium':
            return constants.ERROR
        return constants.WARNING

    def doCheck(self, DSObj, many=False):
        """Perform a healthcheck on a specific DS/lib389 class.  First
        we need to set up the proper args and dicts to properly connect
        to the LDAP server via lib389.  Then run the classes' lint
        functions.

        :param DSObj: a class from lib389 that has built-in lint functions
                      like: Backends, Replica, Encryption, NssSsl, Config, etc
        :returns: a list of Result objects
        """
        args = DSArgs(self.serverid)
        dsrc_inst = {
            'uri': args.instance,
            'basedn': None,
            'binddn': None,
            'bindpw': None,
            'saslmech': None,
            'tls_cacertdir': None,
            'tls_cert': None,
            'tls_key': None,
            'tls_reqcert': 1,
            'starttls': False,
            'prompt': False,
            'pwdfile': None,
            'args': {}
        }
        dsrc_inst['args'][SER_LDAP_URL] = dsrc_inst['uri']
        dsrc_inst['args'][SER_ROOT_DN] = dsrc_inst['binddn']

        inst = connect_instance(dsrc_inst=dsrc_inst, verbose=False, args=args)
        ds_obj = DSObj(inst)
        results = []
        if many:
            # DS class that has many instances of itself (e.g. Backends)
            for clo in ds_obj.list():
                result = clo.lint()
                if result is not None:
                    # DS result could be a single or multiple results
                    if isinstance(result, list):
                        for single_result in result:
                            results += single_result
                    else:
                        results += result
        else:
            # Single object always returns a list of results
            results = ds_obj.lint()
        hc_results = []
        if results is not None:
            for result in results:
                hc_results.append(Result(self,
                                         self.convertSev(result['severity']),
                                         key=result['dsle'],
                                         items=result['items'],
                                         msg=result['detail']))
        disconnect_instance(inst)
        return hc_results

    @duration
    def check(self):
        results = self.doCheck(self.check_class, self.many)
        if len(results) > 0:
            for result in results:
                yield result
        else:
            yield Result(self, constants.SUCCESS)


class DSRegistry(Registry):
    def initialize(self, framework, config, options=None):
        super().initialize(framework, config)
        installutils.check_server_configuration()
        if not api.isdone('bootstrap'):
            api.bootstrap(in_server=True,
                          context='ipahealthcheck',
                          log=None)
        if not api.isdone('finalize'):
            api.finalize()


registry = DSRegistry()



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

from util import raises
from ipahealthcheck.core.plugin import Plugin, Registry


def test_Registry():
    """
    Test the `ipahealthcheck.core.Registry` class
    """
    class plugin1(Plugin):
        pass

    class plugin2(Plugin):
        pass

    # Create a registry
    r = Registry()

    # Check that TypeError is raised trying to register something that isn't
    # a class:
    p = plugin1(r)
    e = raises(TypeError, r, p)
    assert str(e) == 'plugin must be callable; got %r' % p

    # Register the plugins
    r(plugin1)
    r(plugin2)

    # TODO: enforce plugin uniqueness

    # Test registration
    names = [plugin.__class__.__name__ for plugin in r.get_plugins()]
    assert (names == ['plugin1', 'plugin2'])




# Copyright (C) 2022 FreeIPA Contributors see COPYING for license
#

import logging
import os

from ipahealthcheck.ipa.plugin import IPAPlugin, registry
from ipahealthcheck.core.plugin import Result, duration
from ipahealthcheck.core import constants


logger = logging.getLogger()
SYSCONFIG = '/etc/sysconfig/krb5kdc'


def get_contents(file):
    with open(SYSCONFIG, "r") as fd:
        lines = fd.readlines()
    return lines


@registry
class KDCWorkersCheck(IPAPlugin):
    """Verify that the number of workers matches the number of cores"""

    @duration
    def check(self):
        key = 'workers'
        cpus = os.sysconf('SC_NPROCESSORS_ONLN')
        logging.debug('Detected %s CPUs', cpus)

        lines = get_contents(SYSCONFIG)

        args_read = False
        for line in lines:
            sline = line.strip()
            workers = 0
            if sline.startswith('KRB5KDC_ARGS'):
                args_read = True
                sline = sline.split('=', maxsplit=1)[1]
                if sline.find("-w") == -1:
                    if cpus == 1:
                        # -w is not configured when cpus == 1
                        yield Result(self, constants.SUCCESS, key=key)
                        return
                    else:
                        yield Result(self, constants.WARNING, key=key,
                                     sysconfig=SYSCONFIG,
                                     msg='No KDC workers defined in '
                                     '{sysconfig}')
                        return

                # Making an assumption that this line is not misconfigured
                # otherwise the KDC wouldn't start at all.
                sline = sline.replace("'", "")
                sline = sline.replace('"', "")
                sline = sline.split()
                for i in range(len(sline)):
                    if sline[i] == '-w':
                        workers = int(sline[i+1])
                        break
                if cpus == workers:
                    yield Result(self, constants.SUCCESS, key=key)
                else:
                    yield Result(self, constants.WARNING, key=key,
                                 cpus=cpus, workers=workers,
                                 sysconfig=SYSCONFIG,
                                 msg='The number of CPUs {cpus} does not '
                                     'match the number of workers '
                                     '{workers} in {sysconfig}')
                break
        if not args_read:
            yield Result(self, constants.WARNING, key=key,
                         sysconfig=SYSCONFIG,
                         msg='KRB5KDC_ARGS is not set in '
                         '{sysconfig}')



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

from os import environ
import sys

from ipahealthcheck.core import constants
from ipahealthcheck.core.core import RunChecks

try:
    from ipalib.facts import is_ipa_configured
except ImportError:
    is_ipa_configured = None


class IPAChecks(RunChecks):
    def pre_check(self):
        if is_ipa_configured is None:
            print("IPA server is not installed")
            return 1

        if not is_ipa_configured():
            print("IPA server is not configured")
            return 1

        return None

    def add_options(self):
        parser = self.parser
        parser.add_argument('--input-file', dest='infile',
                            help='File to read as input')
        parser.add_argument('--failures-only', dest='failures_only',
                            action='store_true', default=False,
                            help='Exclude SUCCESS results on output (see '
                            'man page for more details)')
        parser.add_argument('--all', dest='all',
                            action='store_true', default=False,
                            help='Report all results on output')
        parser.add_argument('--severity', dest='severity', action="append",
                            help='Include only the selected severity(s)',
                            choices=list(constants._nameToLevel))


def main():
    environ["KRB5_CLIENT_KTNAME"] = "/etc/krb5.keytab"
    environ["KRB5CCNAME"] = "MEMORY:"

    ipachecks = IPAChecks(['ipahealthcheck.registry',
                           'pkihealthcheck.registry'],
                          constants.CONFIG_FILE)
    sys.exit(ipachecks.run_healthcheck())



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

import logging

from ipahealthcheck.core import constants
from ipahealthcheck.core.plugin import Result, duration
from ipahealthcheck.core.service import ServiceCheck
from ipahealthcheck.meta.plugin import registry
try:
    from ipapython.ipaldap import realm_to_serverid
except ImportError:
    from ipaserver.install.installutils import realm_to_serverid

from ipalib import api
from ipaplatform import services
from ipaserver.install import bindinstance
from ipaserver.install import cainstance

logger = logging.getLogger()


class IPAServiceCheck(ServiceCheck):
    @duration
    def check(self, instance=''):
        try:
            # services named with a hyphen cannot be addressed
            # as knownservices.name
            # so use knownservices['name'] instead
            self.service = services.knownservices[self.service_name]
        except KeyError:
            logger.debug(
                "Service '%s' is unknown to ipaplatform, skipping check",
                self.service_name
            )
            return ()

        status = self.service.is_running(instance)

        if status is False:
            yield Result(self, constants.ERROR,
                         status=status, msg='%s: not running' %
                         self.service.service_name)

        else:
            yield Result(self, constants.SUCCESS,
                         status=status)

        return None


@registry
class certmonger(IPAServiceCheck):
    def check(self, instance=''):
        self.service_name = 'certmonger'

        return super().check()


@registry
class dirsrv(IPAServiceCheck):
    def check(self, instance=''):
        self.service_name = 'dirsrv'

        return super().check(realm_to_serverid(api.env.realm))


@registry
class gssproxy(IPAServiceCheck):
    def check(self, instance=''):
        self.service_name = 'gssproxy'

        return super().check()


@registry
class httpd(IPAServiceCheck):
    def check(self, instance=''):
        self.service_name = 'httpd'

        return super().check()


@registry
class ipa_custodia(IPAServiceCheck):
    def check(self, instance=''):
        self.service_name = 'ipa-custodia'

        return super().check()


@registry
class ipa_dnskeysyncd(IPAServiceCheck):
    requires = ('dirsrv',)

    def check(self, instance=''):
        self.service_name = 'ipa-dnskeysyncd'

        if not bindinstance.named_conf_exists():
            return ()

        return super().check()


@registry
class ipa_otpd(IPAServiceCheck):
    def check(self, instance=''):
        self.service_name = 'ipa-otpd'

        return super().check()


@registry
class kadmin(IPAServiceCheck):
    def check(self, instance=''):
        self.service_name = 'kadmin'

        return super().check()


@registry
class krb5kdc(IPAServiceCheck):
    def check(self, instance=''):
        self.service_name = 'krb5kdc'

        return super().check()


@registry
class named(IPAServiceCheck):
    def check(self, instance=''):
        self.service_name = 'named'

        if not bindinstance.named_conf_exists():
            return ()

        return super().check()


@registry
class pki_tomcatd(IPAServiceCheck):
    def check(self, instance=''):
        self.service_name = 'pki_tomcatd'

        ca = cainstance.CAInstance(api.env.realm, host_name=api.env.host)
        if not ca.is_configured():
            return ()

        return super().check()


@registry
class sssd(IPAServiceCheck):
    def check(self, instance=''):
        self.service_name = 'sssd'

        return super().check()



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

import logging

from ipahealthcheck.ipa.plugin import IPAPlugin, registry
from ipahealthcheck.core.plugin import Result
from ipahealthcheck.core.plugin import duration
from ipahealthcheck.core import constants

from ipalib import api

logger = logging.getLogger()


@registry
class IPATopologyDomainCheck(IPAPlugin):
    """
    Execute the equivalant of ipa topologysuffix-verify domain

    Return any errors discovered. This can include:
      * too many agreements
      * connection errors
    """
    def report_errors(self, suffix, result):
        if result['result']['in_order']:
            yield Result(self, constants.SUCCESS, suffix=suffix)
        else:
            max_agmts = result['result']['max_agmts']
            connect_errors = result['result']['connect_errors']
            max_agmts_errors = result['result']['max_agmts_errors']
            cmsg = 'Server %(srv)s can\'t contact servers: %(replicas)s'
            mmsg = 'Server "%(srv)s" has %(n)d agreements, recommended ' \
                   'max %(m)d'

            if connect_errors:
                for error in connect_errors:
                    msg = cmsg % {'srv': error[0],
                                  'replicas': ', '.join(error[1])}
                    yield Result(self, constants.ERROR,
                                 key=error[0],
                                 replicas=error[2],
                                 suffix=suffix,
                                 type='connect',
                                 msg=msg)
            if max_agmts_errors:
                for error in max_agmts_errors:
                    msg = mmsg % {'srv': error[0],
                                  'n': len(error[1]),
                                  'm': max_agmts}
                    yield Result(self, constants.ERROR,
                                 key=error[0],
                                 replicas=error[1],
                                 suffix=suffix,
                                 type='max',
                                 msg=msg)

    def run_check(self, suffix):
        try:
            result = api.Command.topologysuffix_verify(suffix)
        except Exception as e:
            yield Result(self, constants.ERROR,
                         msg='topologysuffix-verify domain failed, %s' %
                         e)
        else:
            for r in self.report_errors(suffix, result):
                yield r

    requires = ('dirsrv',)

    @duration
    def check(self):

        for y in self.run_check(u'domain'):
            yield y
        if api.Command.ca_is_enabled()['result']:
            for y in self.run_check(u'ca'):
                yield y



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

import logging

from ipahealthcheck.dogtag.plugin import DogtagPlugin, registry
from ipahealthcheck.core.plugin import Result
from ipahealthcheck.core.plugin import duration
from ipahealthcheck.core import constants

from ipalib import api, errors, x509
from ipaplatform.paths import paths
from ipaserver.install import certs
from ipaserver.install import ca
from ipaserver.install import krainstance
from ipapython.directivesetter import get_directive
from ipapython.dn import DN
from cryptography.hazmat.primitives.serialization import Encoding

logger = logging.getLogger()


@registry
class DogtagCertsConfigCheck(DogtagPlugin):
    """
    Compare the cert blob in the NSS database to that stored in CS.cfg
    """
    @duration
    def check(self):
        if not self.ca.is_configured():
            logger.debug("No CA configured, skipping dogtag config check")
            return

        kra = krainstance.KRAInstance(api.env.realm)

        blobs = {'auditSigningCert cert-pki-ca': 'ca.audit_signing.cert',
                 'ocspSigningCert cert-pki-ca': 'ca.ocsp_signing.cert',
                 'caSigningCert cert-pki-ca': 'ca.signing.cert',
                 'subsystemCert cert-pki-ca': 'ca.subsystem.cert',
                 'Server-Cert cert-pki-ca': 'ca.sslserver.cert'}

        # Nicknames to skip because their certs are not in CS.cfg
        skip = []

        if kra.is_installed():
            kra_blobs = {
                'transportCert cert-pki-kra':
                'ca.connector.KRA.transportCert',
            }
            blobs.update(kra_blobs)
            skip.append('storageCert cert-pki-kra')
            skip.append('auditSigningCert cert-pki-kra')

        db = certs.CertDB(api.env.realm, paths.PKI_TOMCAT_ALIAS_DIR)
        for nickname, _trust_flags in db.list_certs():
            if nickname in skip:
                logger.debug('Skipping nickname %s because it isn\'t in '
                             'the configuration file')
                continue
            try:
                val = get_directive(paths.CA_CS_CFG_PATH,
                                    blobs[nickname], '=')
            except KeyError:
                logger.debug("%s not found, assuming 3rd party", nickname)
                continue
            if val is None:
                yield Result(self, constants.ERROR,
                             key=nickname,
                             configfile=paths.CA_CS_CFG_PATH,
                             msg='Certificate %s not found in %s' %
                             (blobs[nickname], paths.CA_CS_CFG_PATH))
                continue
            cert = db.get_cert_from_db(nickname)
            pem = cert.public_bytes(Encoding.PEM).decode()
            pem = pem.replace('\n', '')
            pem = pem.replace('-----BEGIN CERTIFICATE-----', '')
            pem = pem.replace('-----END CERTIFICATE-----', '')

            if pem.strip() != val:
                yield Result(self, constants.ERROR,
                             key=nickname,
                             directive=blobs[nickname],
                             configfile=paths.CA_CS_CFG_PATH,
                             msg='Certificate \'%s\' does not match the value '
                             'of %s in %s' %
                             (nickname, blobs[nickname], paths.CA_CS_CFG_PATH))
            else:
                yield Result(self, constants.SUCCESS,
                             key=nickname,
                             configfile=paths.CA_CS_CFG_PATH)


@registry
class DogtagCertsConnectivityCheck(DogtagPlugin):
    """
    Test basic connectivity by using cert-show to fetch a cert
    """
    requires = ('dirsrv',)

    @duration
    def check(self):
        if not self.ca.is_configured():
            logger.debug('CA is not configured, skipping connectivity check')
            return

        config = api.Command.config_show()

        subject_base = config['result']['ipacertificatesubjectbase'][0]
        ipa_subject = ca.lookup_ca_subject(api, subject_base)
        try:
            certs = x509.load_certificate_list_from_file(paths.IPA_CA_CRT)
        except Exception as e:
            yield Result(self, constants.ERROR,
                         key='ipa_ca_crt_file_missing',
                         path=paths.IPA_CA_CRT,
                         error=str(e),
                         msg='The IPA CA cert file {path} could not be '
                             'opened: {error}')
            return

        found = False
        for cert in certs:
            if DN(cert.subject) == ipa_subject:
                found = True
                break

        if not found:
            yield Result(self, constants.ERROR,
                         key='ipa_ca_cert_not_found',
                         subject=str(ipa_subject),
                         path=paths.IPA_CA_CRT,
                         msg='The CA certificate with subject {subject} '
                             'was not found in {path}')
            return
        # Load the IPA CA certificate to obtain its serial number. This
        # was traditionally 1 prior to random serial number support.
        # There is nothing special about cert 1. Even if there is no cert
        # serial number 1 but the connection is ok it is considered passing.
        try:
            api.Command.cert_show(cert.serial_number, all=True)
        except errors.CertificateOperationError as e:
            if 'not found' in str(e):
                yield Result(self, constants.ERROR,
                             key='cert_show_1',
                             error=str(e),
                             serial=str(cert.serial_number),
                             msg='Serial number not found: {error}')
            else:
                yield Result(self, constants.ERROR,
                             key='cert_show_1',
                             error=str(e),
                             serial=str(cert.serial_number),
                             msg='Request for certificate failed: {error}')
        except Exception as e:
            yield Result(self, constants.ERROR,
                         key='cert_show_1',
                             error=str(e),
                         serial=str(cert.serial_number),
                         msg='Request for certificate failed: {error')
        else:
            yield Result(self, constants.SUCCESS)



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

from unittest.mock import Mock, patch
from util import capture_results, CAInstance
from util import m_api

from base import BaseTest
from ipahealthcheck.core import constants, config
from ipahealthcheck.dogtag.plugin import registry
from ipahealthcheck.dogtag.ca import DogtagCertsConnectivityCheck

from ipalib.errors import CertificateOperationError
from ipaplatform.paths import paths
from ipapython.dn import DN


class IPACertificate:
    def __init__(self, serial_number=1,
                 subject='CN=Certificate Authority, O=%s' % m_api.env.realm):
        self.serial_number = serial_number
        self.subject = subject

    def __eq__(self, other):
        return self.serial_number == other.serial_number

    def __hash__(self):
        return hash(self.serial_number)


subject_base = [{
    'result':
        {
            'ipacertificatesubjectbase': [f'O={m_api.env.realm}'],
        },
}]

bad_subject_base = [{
    'result':
        {
            'ipacertificatesubjectbase': ['O=BAD'],
        },
}]


class TestCAConnectivity(BaseTest):
    patches = {
        'ipaserver.install.cainstance.CAInstance':
        Mock(return_value=CAInstance()),
    }

    @patch('ipaserver.install.ca.lookup_ca_subject')
    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_ca_connection_ok(self, mock_load_cert, mock_ca_subject):
        """CA connectivity check when cert_show returns a valid value"""
        m_api.Command.cert_show.side_effect = None
        m_api.Command.config_show.side_effect = subject_base
        m_api.Command.cert_show.return_value = {
            u'result': {u'revoked': False}
        }
        mock_load_cert.return_value = [IPACertificate(12345)]
        mock_ca_subject.return_value = DN(('cn', 'Certificate Authority'),
                                          f'O={m_api.env.realm}')

        framework = object()
        registry.initialize(framework, config.Config)
        f = DogtagCertsConnectivityCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.dogtag.ca'
        assert result.check == 'DogtagCertsConnectivityCheck'

    @patch('ipaserver.install.ca.lookup_ca_subject')
    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_ca_connection_cert_not_found(self, mock_load_cert,
                                          mock_ca_subject):
        """CA connectivity check for a cert that doesn't exist"""
        m_api.Command.cert_show.reset_mock()
        m_api.Command.config_show.side_effect = subject_base
        m_api.Command.cert_show.side_effect = CertificateOperationError(
            message='Certificate operation cannot be completed: '
                    'EXCEPTION (Certificate serial number 0x0 not found)'
        )
        mock_load_cert.return_value = [IPACertificate()]
        mock_ca_subject.return_value = DN(('cn', 'Certificate Authority'),
                                          f'O={m_api.env.realm}')

        framework = object()
        registry.initialize(framework, config.Config)
        f = DogtagCertsConnectivityCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.dogtag.ca'
        assert result.check == 'DogtagCertsConnectivityCheck'
        assert result.kw.get('key') == 'cert_show_1'
        assert result.kw.get('serial') == '1'
        assert result.kw.get('msg') == 'Serial number not found: {error}'

    @patch('ipaserver.install.ca.lookup_ca_subject')
    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_ca_connection_cert_file_not_found(self, mock_load_cert,
                                               mock_ca_subject):
        """CA connectivity check for a cert that doesn't exist"""
        m_api.Command.cert_show.reset_mock()
        m_api.Command.config_show.side_effect = subject_base
        mock_load_cert.side_effect = FileNotFoundError()
        mock_ca_subject.return_value = DN(('cn', 'Certificate Authority'),
                                          f'O={m_api.env.realm}')

        framework = object()
        registry.initialize(framework, config.Config)
        f = DogtagCertsConnectivityCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.dogtag.ca'
        assert result.check == 'DogtagCertsConnectivityCheck'
        assert result.kw.get('key') == 'ipa_ca_crt_file_missing'
        assert result.kw.get('path') == paths.IPA_CA_CRT

    @patch('ipaserver.install.ca.lookup_ca_subject')
    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_ca_connection_cert_not_in_file_list(self, mock_load_cert,
                                                 mock_ca_subject):
        """CA connectivity check for a cert that isn't in IPA_CA_CRT"""
        m_api.Command.cert_show.reset_mock()
        m_api.Command.config_show.side_effect = bad_subject_base
        mock_load_cert.return_value = [IPACertificate()]
        mock_ca_subject.return_value = DN(('cn', 'Certificate Authority'),
                                          'O=BAD')

        framework = object()
        registry.initialize(framework, config.Config)
        f = DogtagCertsConnectivityCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.dogtag.ca'
        assert result.check == 'DogtagCertsConnectivityCheck'
        bad = bad_subject_base[0]['result']['ipacertificatesubjectbase'][0]
        bad_subject = DN(f'CN=Certificate Authority,{bad}')
        assert DN(result.kw['subject']) == bad_subject
        assert result.kw['path'] == paths.IPA_CA_CRT
        assert result.kw['msg'] == (
            'The CA certificate with subject {subject} was not found in {path}'
        )

    @patch('ipaserver.install.ca.lookup_ca_subject')
    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_ca_connection_down(self, mock_load_cert, mock_ca_subject):
        """CA connectivity check with the CA down"""
        m_api.Command.cert_show.side_effect = CertificateOperationError(
            message='Certificate operation cannot be completed: '
                    'Unable to communicate with CMS (503)'
        )
        m_api.Command.config_show.side_effect = subject_base
        mock_load_cert.return_value = [IPACertificate()]
        mock_ca_subject.return_value = DN(('cn', 'Certificate Authority'),
                                          f'O={m_api.env.realm}')

        framework = object()
        registry.initialize(framework, config.Config)
        f = DogtagCertsConnectivityCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.dogtag.ca'
        assert result.check == 'DogtagCertsConnectivityCheck'
        assert result.kw.get('msg') == (
            'Request for certificate failed: {error}'
        )

    @patch('ipaserver.install.ca.lookup_ca_subject')
    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_ca_connection_multiple_ok(self, mock_load_cert, mock_ca_subject):
        """CA connectivity check when cert_show returns a valid value"""
        m_api.Command.cert_show.side_effect = None
        m_api.Command.config_show.side_effect = subject_base
        m_api.Command.cert_show.return_value = {
            u'result': {u'revoked': False}
        }
        mock_load_cert.return_value = [
            IPACertificate(1, 'CN=something'),
            IPACertificate(12345),
        ]
        mock_ca_subject.return_value = DN(('cn', 'Certificate Authority'),
                                          f'O={m_api.env.realm}')

        framework = object()
        registry.initialize(framework, config.Config)
        f = DogtagCertsConnectivityCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.dogtag.ca'

    @patch('ipaserver.install.ca.lookup_ca_subject')
    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_ca_connection_multiple_ok_reverse(self, mock_load_cert,
                                               mock_ca_subject):
        """CA connectivity check when cert_show returns a valid value"""
        m_api.Command.cert_show.side_effect = None
        m_api.Command.config_show.side_effect = subject_base
        m_api.Command.cert_show.return_value = {
            u'result': {u'revoked': False}
        }
        mock_load_cert.return_value = [
            IPACertificate(12345),
            IPACertificate(1, 'CN=something'),
        ]
        mock_ca_subject.return_value = DN(('cn', 'Certificate Authority'),
                                          f'O={m_api.env.realm}')

        framework = object()
        registry.initialize(framework, config.Config)
        f = DogtagCertsConnectivityCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.dogtag.ca'

    @patch('ipaserver.install.ca.lookup_ca_subject')
    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_ca_connection_not_found(self, mock_load_cert, mock_ca_subject):
        """CA connectivity check when cert_show returns a valid value"""
        m_api.Command.cert_show.side_effect = None
        m_api.Command.config_show.side_effect = subject_base
        m_api.Command.cert_show.return_value = {
            u'result': {u'revoked': False}
        }
        mock_load_cert.return_value = [
            IPACertificate(1, 'CN=something'),
        ]
        mock_ca_subject.return_value = DN(('cn', 'Certificate Authority'),
                                          f'O={m_api.env.realm}')

        framework = object()
        registry.initialize(framework, config.Config)
        f = DogtagCertsConnectivityCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.dogtag.ca'
        assert result.kw['msg'] == (
            'The CA certificate with subject {subject} was not found in {path}'
        )



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

from util import raises
from ipahealthcheck.core.plugin import Registry, Plugin, Result, Results
from ipahealthcheck.core import constants


def test_Result():
    """
    Test the `ipahealthcheck.plugin.Result` class
    """

    registry = Registry()
    p = Plugin(registry)

    # Standard case of passing plugin to Result
    r = Result(p, constants.SUCCESS)

    kw = dict(key='value')
    r = Result(p, constants.SUCCESS, **kw)

    e = raises(TypeError, Result)
    assert "__init__() missing 2 required positional arguments: " \
           "'plugin' and 'result'" in str(e)

    # Test passing source and check to Result. This is used for loading
    # a previous output.
    try:
        r = Result(None, constants.SUCCESS)
    except TypeError as e:
        assert str(e) == "source and check or plugin must be provided"

    try:
        r = Result(None, constants.SUCCESS, source='test')
    except TypeError as e:
        assert str(e) == "source and check or plugin must be provided"

    try:
        r = Result(None, constants.SUCCESS, check='test')
    except TypeError as e:
        assert str(e) == "source and check or plugin must be provided"

    r = Result(None, constants.SUCCESS, source='test', check='test')

    # Test results
    r = Result(p, constants.SUCCESS)
    results = Results()
    results.add(r)

    assert len(results) == 1

    r = Result(p, constants.CRITICAL)
    results2 = Results()
    results2.add(r)

    assert len(results2) == 1

    results.extend(results2)

    assert len(results) == 2

    output = list(results.output())
    assert len(output) == 2
    for x in output:
        assert x['source'] == 'ipahealthcheck.core.plugin'
        assert x['check'] == 'Plugin'
        assert x['result'] in (constants.getLevelName(constants.SUCCESS),
                               constants.getLevelName(constants.CRITICAL))
        assert len(x['kw']) == 0


def test_getLevel():
    assert constants.getLevel('SUCCESS') == constants.SUCCESS
    assert constants.getLevel('WARNING') == constants.WARNING
    assert constants.getLevel('ERROR') == constants.ERROR
    assert constants.getLevel('CRITICAL') == constants.CRITICAL
    assert constants.getLevel('FOO') == 'FOO'



#
# Copyright (C) 2021 FreeIPA Contributors see COPYING for license
#

import os

from ipapython.ipautil import run
import pytest


def test_version():
    """
    Test the --version option
    """
    output = run(['ipa-healthcheck', '--version'], env=os.environ)
    assert 'ipahealthcheck' in output.raw_output.decode('utf-8')


@pytest.fixture
def python_ipalib_dir(tmpdir):
    ipalib_dir = tmpdir.mkdir("ipalib")
    ipalib_dir.join("__init__.py").write("")

    def _make_facts(configured=None):
        if configured is None:
            module_text = ""
        elif isinstance(configured, bool):
            module_text = f"def is_ipa_configured(): return {configured}"
        else:
            raise TypeError(
                f"'configured' must be None or bool, got '{configured!r}'"
            )

        ipalib_dir.join("facts.py").write(module_text)
        return str(tmpdir)

    return _make_facts


def test_ipa_notinstalled(python_ipalib_dir, monkeypatch):
    """
    Test ipa-healthcheck handles the missing IPA stuff
    """
    monkeypatch.setenv("PYTHONPATH", python_ipalib_dir(configured=None))
    output = run(["ipa-healthcheck"], raiseonerr=False, env=os.environ)
    assert output.returncode == 1
    assert "IPA server is not installed" in output.raw_output.decode("utf-8")


def test_ipa_unconfigured(python_ipalib_dir, monkeypatch):
    """
    Test ipa-healthcheck handles the unconfigured IPA server
    """
    monkeypatch.setenv("PYTHONPATH", python_ipalib_dir(configured=False))
    output = run(["ipa-healthcheck"], raiseonerr=False, env=os.environ)
    assert output.returncode == 1
    assert "IPA server is not configured" in output.raw_output.decode("utf-8")



#
# Copyright (C) 2020 FreeIPA Contributors see COPYING for license
#

import sys

from ipaclustercheck.core.output import output_registry
from ipahealthcheck.core.core import RunChecks


class ClusterChecks(RunChecks):

    def add_options(self):
        parser = self.parser
        parser.add_argument('--directory', dest='dir',
                            help='Directory holding healthcheck logs')

    def validate_options(self):
        super().validate_options()

        if self.options.dir is None:
            print("--directory containing logs to check is required")
            return 1

        return None


def main():
    clusterchecks = ClusterChecks(['ipaclustercheck.registry'],
                                   '/etc/ipa/clustercheck.conf',
                                   output_registry, 'ansible')
    sys.exit(clusterchecks.run_healthcheck())



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

import logging
from ipalib import api, errors
try:
    from ipapython.ipaldap import realm_to_serverid
except ImportError:
    from ipaserver.install.installutils import realm_to_serverid
from ipaserver.install import cainstance
from ipaserver.install import dsinstance
from ipaserver.install import httpinstance
from ipaserver.install import installutils

from ipahealthcheck.core.plugin import Plugin, Registry

logger = logging.getLogger()


class IPAPlugin(Plugin):
    def __init__(self, reg):
        super().__init__(reg)
        self.ca = cainstance.CAInstance(api.env.realm, host_name=api.env.host)
        self.http = httpinstance.HTTPInstance()
        self.ds = dsinstance.DsInstance()
        self.serverid = realm_to_serverid(api.env.realm)
        self.conn = api.Backend.ldap2


class IPARegistry(Registry):
    def __init__(self):
        super().__init__()
        self.trust_agent = False
        self.trust_controller = False
        self.ca_configured = False

    def initialize(self, framework, config, options=None):
        super().initialize(framework, config)
        # deferred import for mock
        # pylint: disable=import-outside-toplevel
        from ipaserver.servroles import ADtrustBasedRole, ServiceBasedRole
        # pylint: enable=import-outside-toplevel

        installutils.check_server_configuration()

        if not api.isdone('finalize'):
            if not api.isdone('bootstrap'):
                api.bootstrap(in_server=True,
                              context='ipahealthcheck',
                              log=None)
            if not api.isdone('finalize'):
                api.finalize()

        if not api.Backend.ldap2.isconnected():
            try:
                api.Backend.ldap2.connect()
            except (errors.CCacheError, errors.NetworkError) as e:
                logger.debug('Failed to connect to LDAP: %s', e)
            return

        ca = cainstance.CAInstance(api.env.realm, host_name=api.env.host)
        self.ca_configured = ca.is_configured()

        # This package is pulled in when the trust package is installed
        # and is required to lookup trust users. If this is not installed
        # then it can be inferred that trust is not enabled.
        try:
            # pylint: disable=unused-import,import-outside-toplevel
            import pysss_nss_idmap  # noqa: F401
            # pylint: enable=unused-import,import-outside-toplevel
        except ImportError:
            return

        roles = (
            ADtrustBasedRole(u"ad_trust_agent_server",
                             u"AD trust agent"),
            ServiceBasedRole(
                u"ad_trust_controller_server",
                u"AD trust controller",
                component_services=['ADTRUST']
            ),
        )
        role = roles[0].status(api)[0]
        if role.get('status') == 'enabled':
            self.trust_agent = True
        role = roles[1].status(api)[0]
        if role.get('status') == 'enabled':
            self.trust_controller = True


registry = IPARegistry()



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

import grp
import os
import pwd

from ipahealthcheck.core import constants
from ipahealthcheck.core.plugin import Result, duration


class FileCheck:
    """Generic check to validate permission and ownership of files

       files is a tuple of tuples. Each tuple consists of:
           (path, expected_perm, expected_owner, expected_group)

       perm is a POSIX ACL as either a string or tuple: e.g. 0440, (0770,).
       owner and group are names or a tuple of names, not uid/gid.

       If owner and/or group are tuples then all names are checked.
       If a match is found that that is the one reported in SUCCESS.
       If it fails then all values are reported.
    """
    def __init__(self):
        self.files = []

    @duration
    def check(self):
        for (path, owner, group, mode) in self.files:
            if not isinstance(owner, tuple):
                owner = tuple((owner,))
            if not isinstance(group, tuple):
                group = tuple((group,))
            if not isinstance(mode, tuple):
                mode = tuple((mode,))
            if not os.path.exists(path):
                for type in ('mode', 'owner', 'group'):
                    key = '%s_%s' % (path.replace('/', '_'), type)
                    yield Result(self, constants.SUCCESS, key=key,
                                 type=type, path=path,
                                 msg='File does not exist')
                continue
            stat = os.stat(path)
            fmode = str(oct(stat.st_mode)[-4:])
            key = '%s_mode' % path.replace('/', '_')
            if fmode not in mode:
                if len(mode) == 1:
                    modes = mode[0]
                else:
                    modes = 'one of {}'.format(','.join(mode))
                if all(m < fmode for m in mode):
                    yield Result(self, constants.WARNING, key=key,
                                 path=path, type='mode', expected=modes,
                                 got=fmode,
                                 msg='Permissions of %s are too permissive: '
                                 '%s and should be %s' %
                                 (path, fmode, modes))
                elif all(m > fmode for m in mode):
                    yield Result(self, constants.ERROR, key=key,
                                 path=path, type='mode', expected=modes,
                                 got=fmode,
                                 msg='Permissions of %s are too restrictive: '
                                 '%s and should be %s' %
                                 (path, fmode, modes))
                else:
                    yield Result(self, constants.ERROR, key=key,
                                 path=path, type='mode', expected=modes,
                                 got=fmode,
                                 msg='Permissions of %s are unexpected: '
                                 '%s and should be %s' %
                                 (path, fmode, modes))
            else:
                yield Result(self, constants.SUCCESS, key=key,
                             type='mode', path=path)

            found = False
            for o in owner:
                fowner = pwd.getpwnam(o)
                if fowner.pw_uid == stat.st_uid:
                    found = True
                    break

            if not found:
                actual = pwd.getpwuid(stat.st_uid)
                key = '%s_owner' % path.replace('/', '_')
                if len(owner) == 1:
                    msg = 'Ownership of %s is %s and should ' \
                          'be %s' % \
                          (path, actual.pw_name, owner[0])
                else:
                    msg = 'Ownership of %s is %s and should ' \
                          'be one of %s' % \
                          (path, actual.pw_name, ','.join(owner))
                owner = ','.join(owner)
                yield Result(self, constants.WARNING, key=key,
                             path=path, type='owner', expected=owner,
                             got=actual.pw_name,
                             msg=msg)
            else:
                yield Result(self, constants.SUCCESS, key=key,
                             type='owner', path=path)

            found = False
            for g in group:
                fgroup = grp.getgrnam(g)
                if fgroup.gr_gid == stat.st_gid:
                    found = True
                    break

            if not found:
                key = '%s_group' % path.replace('/', '_')
                actual = grp.getgrgid(stat.st_gid)
                if len(group) == 1:
                    msg = 'Group of %s is %s and should ' \
                          'be %s' % \
                          (path, actual.gr_name, group[0])
                else:
                    msg = 'Group of %s is %s and should ' \
                          'be one of %s' % \
                          (path, actual.gr_name, ','.join(group))
                group = ','.join(group)
                yield Result(self, constants.WARNING, key=key,
                             path=path, type='group', expected=group,
                             got=actual.gr_name,
                             msg=msg)
            else:
                yield Result(self, constants.SUCCESS, key=key,
                             type='group', path=path)



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

import configparser
import logging
import SSSDConfig

from ipahealthcheck.ipa.plugin import IPAPlugin, registry
from ipahealthcheck.core.plugin import Result
from ipahealthcheck.core.plugin import duration
from ipahealthcheck.core import constants

from ipalib import api
from ipaplatform.paths import paths
from ipapython import ipautil
from ipapython.dn import DN

try:
    import pysss_nss_idmap
except ImportError:
    # agent and controller will be set to False in init, all tests will
    # be skipped
    pass
try:
    from ipaserver.masters import ENABLED_SERVICE, HIDDEN_SERVICE
except ImportError:
    from ipaserver.install.service import ENABLED_SERVICE, HIDDEN_SERVICE
try:
    from ipapython.ipaldap import realm_to_serverid
except ImportError:
    from ipaserver.install.installutils import realm_to_serverid

logger = logging.getLogger()


def get_trust_domains():
    """
    Get the list of AD trust domains from IPA

    The caller is expected to catch any exceptions.

    Each entry is a dictionary representating an AD domain.
    """
    trust_domains = []
    trusts = api.Command.trust_find(pkey_only=True, raw=True)
    for trust in trusts['result']:
        for cn in trust.get('cn'):
            trustdomains = api.Command.trustdomain_find(cn, raw=True)
            for trustdomain in trustdomains['result']:
                domain = dict()
                domain['domain'] = trustdomain.get('cn')[0]
                domain['domainsid'] = trustdomain.get(
                    'ipanttrusteddomainsid')[0]
                domain['netbios'] = trustdomain.get('ipantflatname')[0]
                trust_domains.append(domain)

    return trust_domains


@registry
class IPATrustAgentCheck(IPAPlugin):
    """
    Check the values that should be set when configures as a trust agent.
    """
    @duration
    def check(self):
        if not self.registry.trust_agent:
            logger.debug('Not a trust agent, skipping')
            return

        try:
            sssdconfig = SSSDConfig.SSSDConfig()
            sssdconfig.import_config()
        except Exception as e:
            logger.debug('Failed to parse sssd.conf: %s', e)
            yield Result(self, constants.CRITICAL, error=str(e),
                         msg='Unable to parse sssd.conf: {error}')
            return
        else:
            domains = sssdconfig.list_active_domains()

        errors = False
        for name in domains:
            domain = sssdconfig.get_domain(name)
            try:
                provider = domain.get_option('id_provider')
            except SSSDConfig.NoOptionError:
                continue
            if provider == "ipa":
                try:
                    mode = domain.get_option('ipa_server_mode')
                except SSSDConfig.NoOptionError:
                    yield Result(self, constants.ERROR,
                                 key='ipa_server_mode_missing',
                                 attr='ipa_server_mode',
                                 domain=name,
                                 sssd_config=paths.SSSD_CONF,
                                 msg='{sssd_config} is missing {attr} '
                                     'in the domain {domain}')
                    errors = True
                else:
                    if not mode:
                        yield Result(self, constants.ERROR,
                                     key='ipa_server_mode_false',
                                     attr='ipa_server_mode',
                                     domain=name,
                                     sssd_config=paths.SSSD_CONF,
                                     msg='{attr} is not True in {sssd_config} '
                                         'in the domain {domain}')
                        errors = True

            if not errors:
                yield Result(self, constants.SUCCESS)


@registry
class IPATrustDomainsCheck(IPAPlugin):
    """
    Check the trust domains
    """
    @duration
    def check(self):
        if not self.registry.trust_agent:
            logger.debug('Not a trust agent, skipping')
            return

        result = ipautil.run([paths.SSSCTL, "domain-list"], raiseonerr=False,
                             capture_output=True)
        if result.returncode != 0:
            yield Result(self, constants.ERROR,
                         key='domain_list_error',
                         sssctl=paths.SSSCTL,
                         error=result.error_log,
                         msg='Execution of {sssctl} failed: {error}')
            return
        sssd_domains = result.output.strip().split('\n')
        if 'implicit_files' in sssd_domains:
            sssd_domains.remove('implicit_files')

        trust_domains = []
        try:
            domains = get_trust_domains()
        except Exception as e:
            yield Result(self, constants.WARNING,
                         key='trust-find',
                         error=str(e),
                         msg='Execution of {key} failed: {error}')
        else:
            for entry in domains:
                trust_domains.append(entry.get('domain'))

        if api.env.domain in sssd_domains:
            sssd_domains.remove(api.env.domain)
        else:
            yield Result(self, constants.ERROR,
                         key=api.env.domain,
                         sssctl=paths.SSSCTL,
                         msg='{key} not in {sssctl} domain-list')

        trust_domains_out = ', '.join(trust_domains)
        sssd_domains_out = ', '.join(sssd_domains)

        if set(trust_domains).symmetric_difference(set(sssd_domains)):
            yield Result(self, constants.ERROR,
                         key='domain-list',
                         sssctl=paths.SSSCTL,
                         sssd_domains=sssd_domains_out,
                         trust_domains=trust_domains_out,
                         msg='{sssctl} {key} reports mismatch: '
                         'sssd domains {sssd_domains} '
                         'trust domains {trust_domains}')
        else:
            yield Result(self, constants.SUCCESS,
                         key='domain-list',
                         sssd_domains=sssd_domains_out,
                         trust_domains=trust_domains_out)

        for domain in sssd_domains:
            args = [paths.SSSCTL, "domain-status", domain, "--online"]
            try:
                result = ipautil.run(args, capture_output=True)
            except Exception as e:
                yield Result(self, constants.WARNING,
                             key='domain-status',
                             error=str(e),
                             msg='Execution of {key} failed: {error}')
                continue
            else:
                if result.output.strip() != 'Online status: Online':
                    yield Result(self, constants.WARNING,
                                 key='domain-status',
                                 domain=domain,
                                 msg='Domain {domain} is not online')
                else:
                    yield Result(self, constants.SUCCESS,
                                 key='domain-status',
                                 domain=domain)


@registry
class IPADomainCheck(IPAPlugin):
    """
    Check that the IPA domain provider is configured to use ipa
    """
    @duration
    def check(self):
        try:
            sssdconfig = SSSDConfig.SSSDConfig()
            sssdconfig.import_config()
        except Exception as e:
            logger.debug('Failed to parse sssd.conf: %s', e)
            yield Result(self, constants.CRITICAL, error=str(e),
                         key='domain-check',
                         msg='Unable to parse sssd.conf: {error}')
            return

        try:
            domain = sssdconfig.get_domain(api.env.domain)
        except SSSDConfig.NoDomainError:
            yield Result(self, constants.ERROR,
                         key='domain-check',
                         domain=api.env.domain,
                         msg='IPA domain {domain} not found in sssd.conf')
            return

        error = False
        for option in ('id_provider', 'auth_provider', 'chpass_provider',
                       'access_provider'):
            try:
                provider = domain.get_option(option)
            except SSSDConfig.NoOptionError:
                yield Result(self, constants.ERROR,
                             key='domain-check',
                             domain=api.env.domain,
                             option=option,
                             msg='Option {option} in domain {domain} not '
                                 'found in sssd.conf')
                error = True
                continue

            if provider != "ipa":
                yield Result(self, constants.ERROR,
                             key='domain-check',
                             option=option,
                             provider=provider,
                             domain=api.env.domain,
                             msg='Option {option} in domain {domain} is '
                                 '{provider} not ipa')
                error = True

        if not error:
            yield Result(self, constants.SUCCESS,
                         key='domain-check')


@registry
class IPATrustCatalogCheck(IPAPlugin):
    """
    Resolve an AD user

    This should populate the 'AD Global catalog' and 'AD Domain Controller'
    fields in 'sssctl domain-status' output (means SSSD actually talks to AD
    DCs)
    """
    @duration
    def check(self):
        if not self.registry.trust_agent:
            logger.debug('Not a trust agent, skipping')
            return

        try:
            trust_domains = get_trust_domains()
        except Exception as e:
            yield Result(self, constants.WARNING,
                         key='trust-find',
                         error=str(e),
                         msg='Execution of {key} failed: {error}')
            trust_domains = []

        for trust_domain in trust_domains:
            sid = trust_domain.get('domainsid')
            try:
                id = pysss_nss_idmap.getnamebysid(sid + '-500')
            except Exception as e:
                yield Result(self, constants.ERROR,
                             key=sid,
                             error=str(e),
                             msg='Look up of{key} failed: {error}')
                continue

            if not id:
                yield Result(self, constants.WARNING,
                             key=sid,
                             error='returned nothing',
                             msg='Look up of {key} {error}')
            else:
                yield Result(self, constants.SUCCESS,
                             key='Domain Security Identifier',
                             sid=sid)

            domain = trust_domain.get('domain')
            args = [paths.SSSCTL, "domain-status", domain, "--active-server"]
            try:
                result = ipautil.run(args, capture_output=True)
            except Exception as e:
                yield Result(self, constants.ERROR,
                             key='domain-status',
                             error=str(e),
                             msg='Execution of {key} failed: {error}')
                continue
            else:
                for txt in ['AD Global Catalog', 'AD Domain Controller']:
                    if txt not in result.output:
                        yield Result(self, constants.ERROR,
                                     key=txt,
                                     output=result.output.strip(),
                                     sssctl=paths.SSSCTL,
                                     domain=domain,
                                     msg='{key} not found in {sssctl} '
                                     '\'domain-status\' output: {output}')
                    else:
                        yield Result(self, constants.SUCCESS,
                                     key=txt,
                                     domain=domain)


@registry
class IPAsidgenpluginCheck(IPAPlugin):
    """
    Verify that the sidgen 389-ds plugins are enabled
    """
    @duration
    def check(self):
        if not self.registry.trust_agent:
            logger.debug('Not a trust agent, skipping')
            return

        for plugin in ['IPA SIDGEN', 'ipa-sidgen-task']:
            sidgen_dn = DN(('cn', plugin), "cn=plugins,cn=config")
            try:
                entry = self.conn.get_entry(
                    sidgen_dn,
                    attrs_list=['nsslapd-pluginEnabled'])
            except Exception as e:
                yield Result(self, constants.ERROR,
                             key=plugin,
                             error=str(e),
                             msg='Error retrieving 389-ds plugin {key}: '
                             '{error}')
            else:
                enabled = entry.get('nsslapd-pluginEnabled', [])
                if len(enabled) != 1:
                    yield Result(self, constants.ERROR,
                                 key=plugin,
                                 dn=str(sidgen_dn),
                                 attr=enabled,
                                 msg='{key}: unexpected value in '
                                 'nsslapd-pluginEnabled in entry {dn}'
                                 '{attr}')
                    continue
                if entry.get('nsslapd-pluginEnabled', [])[0].lower() != 'on':
                    yield Result(self, constants.ERROR,
                                 key=plugin,
                                 msg='389-ds plugin {key} is not enabled')
                else:
                    yield Result(self, constants.SUCCESS,
                                 key=plugin)


@registry
class IPATrustAgentMemberCheck(IPAPlugin):
    """
    Verify that the current host is a member of adtrust agents
    """
    @duration
    def check(self):
        if not self.registry.trust_agent:
            logger.debug('Not a trust agent, skipping')
            return

        agent_dn = DN(('fqdn', api.env.host), api.env.container_host,
                      api.env.basedn)
        group_dn = DN(('cn', 'adtrust agents'), api.env.container_sysaccounts,
                      api.env.basedn)
        try:
            entry = self.conn.get_entry(
                agent_dn,
                attrs_list=['memberOf'])
        except Exception as e:
            yield Result(self, constants.ERROR,
                         key=str(agent_dn),
                         error=str(e),
                         msg='Error retrieving ldap entry {key}: '
                         '{error}')
        else:
            memberof = entry.get('memberof', [])
            for member in memberof:
                if DN(member) == group_dn:
                    yield Result(self, constants.SUCCESS,
                                 key=api.env.host)
                    return

            yield Result(self, constants.ERROR,
                         key=api.env.host,
                         group='adtrust agents',
                         msg='{key} is not a member of {group}')


@registry
class IPATrustControllerPrincipalCheck(IPAPlugin):
    """
    Verify that the current host cifs principal is a member of adtrust agents
    """
    @duration
    def check(self):
        if not self.registry.trust_controller:
            logger.debug('Not a trust controller, skipping')
            return

        agent_dn = DN(('krbprincipalname',
                      'cifs/%s@%s' % (api.env.host, api.env.realm)),
                      api.env.container_service, api.env.basedn)
        group_dn = DN(('cn', 'adtrust agents'), api.env.container_sysaccounts,
                      api.env.basedn)
        try:
            entry = self.conn.get_entry(
                agent_dn,
                attrs_list=['memberOf'])
        except Exception as e:
            yield Result(self, constants.ERROR,
                         key=str(agent_dn),
                         error=str(e),
                         msg='Error retrieving ldap entry {key}: '
                         '{error}')
        else:
            memberof = entry.get('memberof', [])
            for member in memberof:
                if DN(member) == group_dn:
                    yield Result(self, constants.SUCCESS,
                                 key='cifs/%s@%s' %
                                 (api.env.host, api.env.realm))
                    return

            yield Result(self, constants.ERROR,
                         key='cifs/%s@%s' % (api.env.host, api.env.realm),
                         group='adtrust agents',
                         msg='{key} is not a member of {group}')


@registry
class IPATrustControllerServiceCheck(IPAPlugin):
    """
    Verify that the current host starts the ADTRUST service.
    """
    @duration
    def check(self):
        if not self.registry.trust_controller:
            logger.debug('Not a trust controller, skipping')
            return

        service_dn = DN(('cn', 'ADTRUST'), ('cn', api.env.host),
                        api.env.container_masters, api.env.basedn)

        try:
            entry = self.conn.get_entry(
                service_dn,
                attrs_list=['ipaconfigstring'])
        except Exception as e:
            yield Result(self, constants.ERROR,
                         key=str(service_dn),
                         error=str(e),
                         msg='Error retrieving ldap entry {key}: '
                         '{error}')
        else:
            configs = entry.get('ipaconfigstring', [])
            enabled = False
            for config in configs:
                if config in [ENABLED_SERVICE, HIDDEN_SERVICE]:
                    enabled = True
                    break

            if enabled:
                yield Result(self, constants.SUCCESS,
                             key='ADTRUST')
            else:
                yield Result(self, constants.ERROR,
                             key='ADTRUST',
                             msg='{key} service is not enabled')


@registry
class IPATrustControllerConfCheck(IPAPlugin):
    """
    Verify that certain elements of the configuration are unchanged

    This is expected to be expanded over time.
    """
    @duration
    def check(self):
        if not self.registry.trust_controller:
            logger.debug('Not a trust controller, skipping')
            return

        ldapi_socket = "ipasam:ldapi://%%2fvar%%2frun%%2fslapd-%s.socket" % \
                       realm_to_serverid(api.env.realm)

        try:
            result = ipautil.run(['net', 'conf', 'list'], capture_output=True)
        except Exception as e:
            yield Result(self, constants.ERROR,
                         key='net conf list',
                         error=str(e),
                         msg='Execution of {key} failed: {error}')
            return

        conf = result.output.replace('\t', '')
        config = configparser.ConfigParser(delimiters=('='),
                                           interpolation=None)
        try:
            config.read_string(conf)
        except Exception as e:
            yield Result(self, constants.ERROR,
                         key='net conf list',
                         error=str(e),
                         msg='Unable to parse {key} output: {error}')
            return

        try:
            net_ldapi = config.get('global', 'passdb backend')
        except Exception as e:
            yield Result(self, constants.ERROR,
                         key='net conf list',
                         error=str(e),
                         section='global',
                         option='passdb backend',
                         msg='Unable to read \'{option}\' in section '
                         '{section} in {key} output: {error}')
            return

        if net_ldapi != ldapi_socket:
            yield Result(self, constants.ERROR,
                         key='net conf list',
                         got=net_ldapi,
                         expected=ldapi_socket,
                         option='passdb backend',
                         msg='{key} option {option} value {got} '
                         'doesn\'t match expected value {expected}')
        else:
            yield Result(self, constants.SUCCESS,
                         key='net conf list')


@registry
class IPATrustControllerGroupSIDCheck(IPAPlugin):
    """
    Verify that the admins group's SID ends with 512 (Domain Admins RID)
    """
    @duration
    def check(self):
        if not self.registry.trust_controller:
            logger.debug('Not a trust controller, skipping')
            return

        admins_dn = DN(('cn', 'admins'),
                       api.env.container_group, api.env.basedn)

        try:
            entry = self.conn.get_entry(
                admins_dn,
                attrs_list=['ipantsecurityidentifier'])
        except Exception as e:
            yield Result(self, constants.ERROR,
                         key=str(admins_dn),
                         error=str(e),
                         msg='Error retrieving ldap entry {key}: '
                         '{error}')
            return

        identifier = entry.get('ipantsecurityidentifier', [None])[0]
        if not identifier or not identifier.endswith('512'):
            yield Result(self, constants.ERROR,
                         key='ipantsecurityidentifier',
                         rid=identifier,
                         msg='{key} is not a Domain Admins RID')
        else:
            yield Result(self, constants.SUCCESS,
                         rid=identifier,
                         key='ipantsecurityidentifier')


@registry
class IPATrustControllerAdminSIDCheck(IPAPlugin):
    """
    Verify that the admin user's SID ends with 500
    """
    @duration
    def check(self):
        if not self.registry.trust_controller:
            logger.debug('Not a trust controller, skipping')
            return

        admin_dn = DN(('uid', 'admin'),
                      api.env.container_user, api.env.basedn)

        try:
            entry = self.conn.get_entry(
                admin_dn,
                attrs_list=['ipantsecurityidentifier'])
        except Exception as e:
            yield Result(self, constants.ERROR,
                         key=str(admin_dn),
                         error=str(e),
                         msg='Error retrieving the admin user at {key}: '
                         '{error}')
            return

        identifier = entry.get('ipantsecurityidentifier', [None])[0]
        if not identifier or not identifier.endswith('500'):
            yield Result(self, constants.ERROR,
                         key='ipantsecurityidentifier',
                         rid=identifier,
                         msg='{key} is not a Domain Admin RID')
        else:
            yield Result(self, constants.SUCCESS,
                         rid=identifier,
                         key='ipantsecurityidentifier')


@registry
class IPATrustPackageCheck(IPAPlugin):
    """
    If AD trust is enabled verify that the trust-ad pkg is installed

    If AD trust is enabled and the master does not have the
    freeipa-server-trust-ad package installed then the master will
    able to resolve users/groups via extdom plugin and sssd but won't
    be able to do framework-specific operations.
    """
    @duration
    def check(self):
        if self.registry.trust_controller:
            logger.debug('Trust controller, skipping')
            return
        if not self.registry.trust_agent:
            logger.debug('Not a trust agent, skipping')
            return

        # The trust-ad package provides this import
        try:
            # pylint: disable=unused-import,import-outside-toplevel
            from ipaserver.install import adtrustinstance  # noqa: F401
            # pylint: enable=unused-import,import-outside-toplevel
            yield Result(self, constants.SUCCESS,
                         key='adtrustpackage')
        except ImportError:
            yield Result(self, constants.WARNING,
                         key='adtrustpackage',
                         msg='trust-ad sub-package is not installed. '
                         'Administration will be limited.')



#
# Copyright (C) 2021 FreeIPA Contributors see COPYING for license
#

import time

from ipahealthcheck.core.plugin import Plugin, Registry, Result
from ipahealthcheck.core.core import run_plugins
from ipahealthcheck.core import constants


def test_timeout():
    """
    Test that timeouts are detected.
    """
    class plugin1(Plugin):
        def check(self):
            time.sleep(5)

    class plugin2(Plugin):
        def check(self):
            yield Result(self, constants.SUCCESS, key='test', msg='pass')

    # Create a registry
    r = Registry()

    # Register the plugins
    r(plugin1)
    r(plugin2)

    # Collect the results
    results = run_plugins(r.get_plugins(), (), None, None, {}, timeout=1)

    assert len(results.results) == 2

    assert results.results[0].result == constants.ERROR
    assert results.results[0].kw.get('exception') == 'Request timed out'

    assert results.results[1].result == constants.SUCCESS
    assert results.results[1].kw.get('msg') == 'pass'



#
# Copyright (C) 2020 FreeIPA Contributors see COPYING for license
#

import argparse

from ipahealthcheck.core.output import output_registry


class RunChecks:
    def run_healthcheck(self):
        options = argparse.Namespace(check=None, debug=False, indent=2,
                                     list_sources=False, output_file=None,
                                     output_type='json', source=None,
                                     verbose=False)

        for out in output_registry.plugins:
            if out.__name__.lower() == options.output_type:
                out(options)
                break


def test_run_healthcheck():
    """
    Test typical initialization in run_healthcheck (based ok pki-healthcheck)
    """
    run = RunChecks()
    run.run_healthcheck()



#
# Copyright (C) 2022 FreeIPA Contributors see COPYING for license
#

from base import BaseTest
from unittest.mock import patch
from util import capture_results

from ipahealthcheck.core import config, constants
from ipahealthcheck.ipa.plugin import registry
from ipahealthcheck.ipa.kdc import KDCWorkersCheck


class TestKDCWorkers(BaseTest):
    @patch('ipahealthcheck.ipa.kdc.get_contents')
    @patch('os.sysconf')
    def test_no_workers_noargs(self, mock_sysconf, mock_sysconfig):
        mock_sysconf.return_value = 1
        mock_sysconfig.return_value = ""
        framework = object()
        registry.initialize(framework, config.Config)
        f = KDCWorkersCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.WARNING
        assert result.source == 'ipahealthcheck.ipa.kdc'
        assert result.check == 'KDCWorkersCheck'
        assert result.kw.get('key') == 'workers'
        assert result.kw.get('sysconfig') == '/etc/sysconfig/krb5kdc'
        assert result.kw.get('msg') == 'KRB5KDC_ARGS is not set in {sysconfig}'

    @patch('ipahealthcheck.ipa.kdc.get_contents')
    @patch('os.sysconf')
    def test_no_workers_empty_noargs(self, mock_sysconf, mock_sysconfig):
        mock_sysconf.return_value = 1
        mock_sysconfig.return_value = ("KRB5KDC_ARGS=",)
        framework = object()
        registry.initialize(framework, config.Config)
        f = KDCWorkersCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.ipa.kdc'
        assert result.check == 'KDCWorkersCheck'
        assert result.kw.get('key') == 'workers'

    @patch('ipahealthcheck.ipa.kdc.get_contents')
    @patch('os.sysconf')
    def test_with_workers_empty_noargs(self, mock_sysconf, mock_sysconfig):
        mock_sysconf.return_value = 2
        mock_sysconfig.return_value = ("KRB5KDC_ARGS=",)
        framework = object()
        registry.initialize(framework, config.Config)
        f = KDCWorkersCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.WARNING
        assert result.source == 'ipahealthcheck.ipa.kdc'
        assert result.check == 'KDCWorkersCheck'
        assert result.kw.get('key') == 'workers'
        assert result.kw.get('sysconfig') == '/etc/sysconfig/krb5kdc'
        assert result.kw.get('msg') == 'No KDC workers defined in {sysconfig}'

    @patch('ipahealthcheck.ipa.kdc.get_contents')
    @patch('os.sysconf')
    def test_workers_match_single(self, mock_sysconf, mock_sysconfig):
        mock_sysconf.return_value = 1
        mock_sysconfig.return_value = (
            ("KRB5KDC_ARGS='-w 1'", "KRB5REALM=EXAMPLE.TEST")
        )
        framework = object()
        registry.initialize(framework, config.Config)
        f = KDCWorkersCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.ipa.kdc'
        assert result.check == 'KDCWorkersCheck'
        assert result.kw.get('key') == 'workers'

    @patch('ipahealthcheck.ipa.kdc.get_contents')
    @patch('os.sysconf')
    def test_workers_match_double(self, mock_sysconf, mock_sysconfig):
        mock_sysconf.return_value = 1
        mock_sysconfig.return_value = (
            ('KRB5KDC_ARGS="-w 1"', "KRB5REALM=EXAMPLE.TEST")
        )
        framework = object()
        registry.initialize(framework, config.Config)
        f = KDCWorkersCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.ipa.kdc'
        assert result.check == 'KDCWorkersCheck'

    @patch('ipahealthcheck.ipa.kdc.get_contents')
    @patch('os.sysconf')
    def test_workers_mismatch(self, mock_sysconf, mock_sysconfig):
        mock_sysconf.return_value = 2
        mock_sysconfig.return_value = (
            ("KRB5KDC_ARGS='-w 1'", "KRB5REALM=EXAMPLE.TEST")
        )
        framework = object()
        registry.initialize(framework, config.Config)
        f = KDCWorkersCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.WARNING
        assert result.source == 'ipahealthcheck.ipa.kdc'
        assert result.check == 'KDCWorkersCheck'
        assert result.kw.get('key') == 'workers'
        assert result.kw.get('cpus') == 2
        assert result.kw.get('workers') == 1
        assert result.kw.get("msg") == "The number of CPUs {cpus} " \
                                       "does not match the number of " \
                                       "workers {workers} in {sysconfig}"



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

import tempfile

import pytest

from ipahealthcheck.core.config import read_config, convert_string


def test_config_no_section():
    with tempfile.NamedTemporaryFile('w') as f:
        f.write('\n')
        f.flush()

        config = read_config(f.name)

    assert config is None


def test_config_bad_format():
    with tempfile.NamedTemporaryFile('w') as f:
        f.write('bad\n')

        config = read_config(f.name)
        f.flush()

    assert config is None


def test_config_values():
    with tempfile.NamedTemporaryFile('w') as f:
        f.write('[default]\nfoo = bar\n')
        f.flush()

        config = read_config(f.name)

    assert config.foo == 'bar'

    with pytest.raises(KeyError):
        config.bar  # pylint: disable=pointless-statement


def test_config_recursion():
    with tempfile.NamedTemporaryFile('w') as f:
        f.write('[default]\nfoo = bar\n')
        f.flush()

        config = read_config(f.name)

    assert config.foo == 'bar'

    # The config dict is in the object
    assert isinstance(config._Config__d, dict)

    # But it isn't recursive
    try:
        config._Config__d['_Config__d']
    except KeyError:
        pass


def test_convert_string():
    for value in ("s", "string", "BiggerString"):
        assert convert_string(value) == value

    for value in ("True", "true", True):
        assert convert_string(value) is True

    for value in ("False", "false", False):
        assert convert_string(value) is False

    for value in ("10", "99999", 807):
        assert convert_string(value) == int(value)



#
# Copyright (C) 2021 FreeIPA Contributors see COPYING for license
#

from base import BaseTest
from io import BytesIO
from unittest.mock import patch
from util import capture_results
import lxml.etree

from ipahealthcheck.core import config, constants
from ipahealthcheck.meta.plugin import registry
from ipahealthcheck.ipa.proxy import IPAProxySecretCheck

# Pre-parse the XML to avoid Mock weirdness
good_xml_input = """
<Server port="8005" shutdown="SHUTDOWN">
   <Connector port="8009" protocol="AJP/1.3" redirectPort="8443" address="127.0.0.1" name="Connector1" secret="somesecret"/>
   <Connector port="8009" protocol="AJP/1.3" redirectPort="8443" address="::1" name="Connector1" secret="somesecret"/>
</Server>
"""  # noqa: E501
good_xml = lxml.etree.parse(BytesIO(good_xml_input.encode('utf-8')))

good_ipa_proxy = """
<LocationMatch "^/ca/ee/ca/checkRequest">
    SSLOptions +StdEnvVars +ExportCertData +StrictRequire +OptRenegotiate
    SSLVerifyClient none
    ProxyPassMatch ajp://localhost:8009 secret=somesecret
    ProxyPassReverse ajp://localhost:8009
</LocationMatch>
"""

empty_ipa_proxy = """
"""

different_secrets_ipa_proxy = """
<LocationMatch "^/ca/ee/ca/checkRequest">
    SSLOptions +StdEnvVars +ExportCertData +StrictRequire +OptRenegotiate
    SSLVerifyClient none
    ProxyPassMatch ajp://localhost:8009 secret=somesecret
    ProxyPassReverse ajp://localhost:8009
</LocationMatch>
<LocationMatch "^/ca/ee/ca/checkRequest">
    SSLOptions +StdEnvVars +ExportCertData +StrictRequire +OptRenegotiate
    SSLVerifyClient none
    ProxyPassMatch ajp://localhost:8009 secret=othersecret
    ProxyPassReverse ajp://localhost:8009
</LocationMatch>
"""

# server.xml secret won't match Apache secret
mismatch1_xml_input = """
<Server port="8005" shutdown="SHUTDOWN">
   <Connector port="8009" protocol="AJP/1.3" redirectPort="8443" address="127.0.0.1" name="Connector1" secret="badsecret"/>
   <Connector port="8009" protocol="AJP/1.3" redirectPort="8443" address="::1" name="Connector1" secret="badsecret"/>
</Server>
"""  # noqa: E501
mismatch1_xml = lxml.etree.parse(BytesIO(mismatch1_xml_input.encode('utf-8')))

both_secrets_xml_input = """
<Server port="8005" shutdown="SHUTDOWN">
   <Connector port="8009" protocol="AJP/1.3" redirectPort="8443" address="127.0.0.1" name="Connector1" secret="somesecret" requiredSecret="somesecret"/>
   <Connector port="8009" protocol="AJP/1.3" redirectPort="8443" address="::1" name="Connector1" secret="somesecret" requiredSecret="somesecret"/>
</Server>
"""  # noqa: E501
both_secrets_xml = lxml.etree.parse(
    BytesIO(both_secrets_xml_input.encode('utf-8'))
)

both_secrets_mismatch_xml_input = """
<Server port="8005" shutdown="SHUTDOWN">
   <Connector port="8009" protocol="AJP/1.3" redirectPort="8443" address="127.0.0.1" name="Connector1" secret="somesecret" requiredSecret="othersecret"/>
   <Connector port="8009" protocol="AJP/1.3" redirectPort="8443" address="::1" name="Connector1" secret="somesecret" requiredSecret="othersecret"/>
</Server>
"""  # noqa: E501
both_secrets_mismatch_xml = lxml.etree.parse(
    BytesIO(both_secrets_mismatch_xml_input.encode('utf-8'))
)

empty_xml_input = """
<Server port="8005" shutdown="SHUTDOWN">
</Server>
"""
empty_xml = lxml.etree.parse(BytesIO(empty_xml_input.encode('utf-8')))


class TestIPAProxySecretCheck(BaseTest):
    @patch('lxml.etree.parse')
    @patch('ipahealthcheck.ipa.proxy.read_ipa_pki_proxy')
    def test_matching_secrets(self, mock_proxy, mock_ltree):
        """The passwords match"""
        mock_ltree.return_value = good_xml
        mock_proxy.return_value = good_ipa_proxy.split('\n')

        framework = object()
        registry.initialize(framework, config.Config())
        f = IPAProxySecretCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS

    @patch('lxml.etree.parse')
    @patch('ipahealthcheck.ipa.proxy.read_ipa_pki_proxy')
    def test_xml_both_secrets(self, mock_proxy, mock_ltree):
        """server.xml defines both secret types and they match"""
        mock_ltree.return_value = both_secrets_xml
        mock_proxy.return_value = good_ipa_proxy.split('\n')

        framework = object()
        registry.initialize(framework, config.Config())
        f = IPAProxySecretCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS

    @patch('lxml.etree.parse')
    @patch('ipahealthcheck.ipa.proxy.read_ipa_pki_proxy')
    def test_xml_both_secret_type_mismatch(self, mock_proxy, mock_ltree):
        """XML has both secret attributes and they do not match"""
        mock_ltree.return_value = both_secrets_mismatch_xml
        mock_proxy.return_value = good_ipa_proxy.split('\n')

        framework = object()
        registry.initialize(framework, config.Config())
        f = IPAProxySecretCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.WARNING
        assert result.kw.get('msg') == 'The AJP secrets in {server_xml} do '\
                                       'not match'

    @patch('lxml.etree.parse')
    @patch('ipahealthcheck.ipa.proxy.read_ipa_pki_proxy')
    def test_xml_secret_mismatch(self, mock_proxy, mock_ltree):
        """The Apache secret doesn't match the tomcat secret"""
        mock_ltree.return_value = mismatch1_xml
        mock_proxy.return_value = good_ipa_proxy.split('\n')

        framework = object()
        registry.initialize(framework, config.Config())
        f = IPAProxySecretCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.CRITICAL
        assert result.kw.get('msg') == 'A ProxyPassMatch secret not found ' \
                                       'in {server_xml}'

    @patch('lxml.etree.parse')
    @patch('ipahealthcheck.ipa.proxy.read_ipa_pki_proxy')
    def test_xml_no_connectors(self, mock_proxy, mock_ltree):
        """No connectors found in server.xml"""
        mock_ltree.return_value = empty_xml
        mock_proxy.return_value = good_ipa_proxy.split('\n')

        framework = object()
        registry.initialize(framework, config.Config())
        f = IPAProxySecretCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.CRITICAL
        assert result.kw.get('msg') == 'No AJP/1.3 Connectors defined in ' \
                                       '{server_xml}'

    @patch('lxml.etree.parse')
    @patch('ipahealthcheck.ipa.proxy.read_ipa_pki_proxy')
    def test_no_proxypassmatch(self, mock_proxy, mock_ltree):
        """No connectors found in server.xml"""
        mock_ltree.return_value = good_xml
        mock_proxy.return_value = empty_ipa_proxy.split('\n')

        framework = object()
        registry.initialize(framework, config.Config())
        f = IPAProxySecretCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.CRITICAL
        assert result.kw.get('msg') == 'No ProxyPassMatch secrets found ' \
                                       'in {proxy_conf}'

    @patch('lxml.etree.parse')
    @patch('ipahealthcheck.ipa.proxy.read_ipa_pki_proxy')
    def test_proxypassmatch_different_secrets(self, mock_proxy, mock_ltree):
        """No connectors found in server.xml"""
        mock_ltree.return_value = good_xml
        mock_proxy.return_value = different_secrets_ipa_proxy.split('\n')

        framework = object()
        registry.initialize(framework, config.Config())
        f = IPAProxySecretCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 2

        result = self.results.results[0]
        assert result.result == constants.CRITICAL
        assert result.kw.get('msg') == 'Not all ProxyPassMatch secrets ' \
                                       'match in {proxy_conf}'

        result = self.results.results[1]
        assert result.result == constants.CRITICAL
        assert result.kw.get('msg') == 'A ProxyPassMatch secret not found ' \
                                       'in {server_xml}'



#
# Copyright (C) 2020 FreeIPA Contributors see COPYING for license
#

from base import BaseTest
from collections import namedtuple
from unittest.mock import patch
from util import capture_results

from ipahealthcheck.core import config, constants
from ipahealthcheck.meta.plugin import registry
from ipahealthcheck.meta.core import MetaCheck
from ipapython import ipautil
from ipaplatform.paths import paths

if 'FIPS_MODE_SETUP' not in dir(paths):
    paths.FIPS_MODE_SETUP = '/usr/bin/fips-mode-setup'


def gen_result(returncode, output='', error=''):
    """
    Generate the result of an execution.

    Creates a run namespace and sets the output as provided.
    """
    run_result = namedtuple(
        'run', ['returncode', 'raw_output', 'output_log', 'error_log']
    )
    run_result.returncode = returncode
    run_result.raw_output = output.encode('utf-8')
    run_result.output_log = output
    run_result.error_log = error

    return run_result


class TestMetaFIPS(BaseTest):
    @patch('os.path.exists')
    def test_fips_no_fips_mode_setup(self, mock_exists):
        mock_exists.return_value = False

        framework = object()
        registry.initialize(framework, config.Config())
        f = MetaCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.meta.core'
        assert result.check == 'MetaCheck'
        assert result.kw.get('fips') == 'missing %s' % paths.FIPS_MODE_SETUP

    @patch('os.path.exists')
    @patch('ipapython.ipautil.run')
    def test_fips_disabled(self, mock_run, mock_exists):
        mock_exists.return_value = True

        mock_run.side_effect = [
            gen_result(2),
            gen_result(0, output='ACME is disabled'),
        ]

        framework = object()
        registry.initialize(framework, config.Config())
        f = MetaCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.meta.core'
        assert result.check == 'MetaCheck'
        assert result.kw.get('fips') == 'disabled'

    @patch('os.path.exists')
    @patch('ipapython.ipautil.run')
    def test_fips_enabled(self, mock_run, mock_exists):
        mock_exists.return_value = True

        mock_run.side_effect = [
            gen_result(0),
            gen_result(0, output='ACME is disabled'),
        ]

        framework = object()
        registry.initialize(framework, config.Config())
        f = MetaCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.meta.core'
        assert result.check == 'MetaCheck'
        assert result.kw.get('fips') == 'enabled'

    @patch('os.path.exists')
    @patch('ipapython.ipautil.run')
    def test_fips_inconsistent(self, mock_run, mock_exists):
        mock_exists.return_value = True

        mock_run.side_effect = [
            gen_result(1),
            gen_result(0, output='ACME is disabled'),
        ]

        framework = object()
        registry.initialize(framework, config.Config())
        f = MetaCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.meta.core'
        assert result.check == 'MetaCheck'
        assert result.kw.get('fips') == 'inconsistent'

    @patch('os.path.exists')
    @patch('ipapython.ipautil.run')
    def test_fips_unknown(self, mock_run, mock_exists):
        mock_exists.return_value = True

        mock_run.side_effect = [
            gen_result(103),
            gen_result(0, output='ACME is disabled'),
        ]

        framework = object()
        registry.initialize(framework, config.Config())
        f = MetaCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.meta.core'
        assert result.check == 'MetaCheck'
        assert result.kw.get('fips') == 'unknown'

    @patch('os.path.exists')
    @patch('ipapython.ipautil.run')
    def test_fips_failed(self, mock_run, mock_exists):
        mock_exists.return_value = True

        mock_run.side_effect = [
            ipautil.CalledProcessError(
                1, 'fips-mode-setup', output='execution failed'
            ),
            gen_result(0, output='ACME is disabled'),
        ]

        framework = object()
        registry.initialize(framework, config.Config())
        f = MetaCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.meta.core'
        assert result.check == 'MetaCheck'
        assert result.kw.get('fips') == 'failed to check'


class TestMetaACME(BaseTest):
    @patch('os.path.exists')
    def test_acme_no_ipa_acme_status(self, mock_exists):
        mock_exists.return_value = False

        framework = object()
        registry.initialize(framework, config.Config())
        f = MetaCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.meta.core'
        assert result.check == 'MetaCheck'
        assert result.kw.get('acme') == \
            'missing %s' % '/usr/sbin/ipa-acme-manage'

    @patch('os.path.exists')
    @patch('ipapython.ipautil.run')
    def test_acme_disabled(self, mock_run, mock_exists):
        mock_exists.return_value = True

        mock_run.side_effect = [
            gen_result(0),
            gen_result(0, output='ACME is disabled'),
        ]

        framework = object()
        registry.initialize(framework, config.Config())
        f = MetaCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.meta.core'
        assert result.check == 'MetaCheck'
        assert result.kw.get('acme') == 'disabled'

    @patch('os.path.exists')
    @patch('ipapython.ipautil.run')
    def test_acme_enabled(self, mock_run, mock_exists):
        mock_exists.return_value = True

        mock_run.side_effect = [
            gen_result(0),
            gen_result(0, output='ACME is enabled'),
        ]

        framework = object()
        registry.initialize(framework, config.Config())
        f = MetaCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.meta.core'
        assert result.check == 'MetaCheck'
        assert result.kw.get('acme') == 'enabled'

    @patch('os.path.exists')
    @patch('ipapython.ipautil.run')
    def test_acme_unknown(self, mock_run, mock_exists):
        mock_exists.return_value = True

        mock_run.side_effect = [
            gen_result(0),
            gen_result(
                0,
                error="cannot connect to 'https://somewhere/acme/login"
            ),
        ]

        framework = object()
        registry.initialize(framework, config.Config())
        f = MetaCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.meta.core'
        assert result.check == 'MetaCheck'
        assert result.kw.get('acme') == 'unknown'



#
# Copyright (C) 2020 FreeIPA Contributors see COPYING for license
#

from util import capture_results, CAInstance
from util import m_api
from base import BaseTest
from unittest.mock import Mock, patch

from ipahealthcheck.core import config, constants
from ipahealthcheck.ipa.plugin import registry
from ipahealthcheck.ipa.certs import IPACertDNSSAN
from mock_certmonger import create_mock_dbus, _certmonger
from mock_certmonger import get_expected_requests, set_requests


class IPACertificate:
    def __init__(self, serial_number=1, no_san=False):
        self.subject = 'CN=%s' % m_api.env.host
        self.issuer = 'CN=ISSUER'
        self.serial_number = serial_number
        self.san_a_label_dns_names = [m_api.env.host]
        if not no_san:
            self.san_a_label_dns_names.append('ipa-ca.%s' % m_api.env.domain)


class TestDNSSAN(BaseTest):
    patches = {
        'ipaserver.install.certs.is_ipa_issued_cert':
        Mock(return_value=True),
        'ipahealthcheck.ipa.certs.get_expected_requests':
        Mock(return_value=get_expected_requests()),
        'ipalib.install.certmonger._cm_dbus_object':
        Mock(side_effect=create_mock_dbus),
        'ipalib.install.certmonger._certmonger':
        Mock(return_value=_certmonger()),
        'ipaserver.install.cainstance.CAInstance':
        Mock(return_value=CAInstance()),
        'socket.getfqdn':
        Mock(return_value=m_api.env.host),
    }

    @patch('ipalib.install.certmonger.get_request_value')
    @patch('ipalib.x509.load_certificate_from_file')
    def test_dnssan_ok(self, mock_cert, mock_value):
        set_requests()

        mock_value.side_effect = ['dogtag-ipa-ca-renew-agent',
                                  'IPA', 'caIPAserviceCert']
        mock_cert.return_value = IPACertificate()

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertDNSSAN(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.kw.get('san') == [m_api.env.host,
                                        'ipa-ca.%s' % m_api.env.domain]
        assert result.kw.get('hostname') == [m_api.env.host,
                                             'ipa-ca.%s' % m_api.env.domain]
        assert result.kw.get('profile') == 'caIPAserviceCert'
        assert result.check == 'IPACertDNSSAN'

    @patch('ipalib.install.certmonger.get_request_value')
    def test_sandns_no_certs(self, mock_value):
        set_requests()

        mock_value.side_effect = ['dogtag-ipa-ca-renew-agent',
                                  'dogtag-ipa-ca-renew-agent']

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertDNSSAN(registry)

        self.results = capture_results(f)

        # No IPA CA, no results
        assert len(self.results) == 0

    @patch('ipalib.install.certmonger.get_request_value')
    @patch('ipalib.x509.load_certificate_from_file')
    def test_dnssan_missing_ipaca(self, mock_cert, mock_value):
        set_requests()

        mock_value.side_effect = ['dogtag-ipa-ca-renew-agent',
                                  'IPA', 'caIPAserviceCert']
        mock_cert.return_value = IPACertificate(no_san=True)

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertDNSSAN(registry)

        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.kw.get('san') == [m_api.env.host]
        assert result.kw.get('hostname') == 'ipa-ca.%s' % m_api.env.domain
        assert result.kw.get('profile') == 'caIPAserviceCert'
        assert result.kw.get('ca') == 'IPA'
        assert result.kw.get('key') == '5678'



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

from util import capture_results, CAInstance, KRAInstance
from base import BaseTest
from ipahealthcheck.core import config, constants
from ipahealthcheck.ipa.plugin import registry
from ipahealthcheck.ipa.certs import IPACertNSSTrust
from ipaplatform.paths import paths
from unittest.mock import Mock, patch


class mock_CertDB:
    def __init__(self, trust):
        """A dict of nickname + NSSdb trust flags"""
        self.trust = trust

    def list_certs(self):
        return [(nickname, self.trust[nickname]) for nickname in self.trust]


class mock_NSSDatabase:
    def __init__(self, nssdir, token=None, pwd_file=None, trust=None):
        self.trust = trust
        self.token = token

    def list_certs(self):
        return [(nickname, self.trust[nickname]) for nickname in self.trust]


def my_unparse_trust_flags(trust_flags):
    return trust_flags


# These tests make some assumptions about the order in which the
# results are returned.

class TestNSSDBTrust(BaseTest):
    patches = {
        'ipaserver.install.krainstance.KRAInstance':
        Mock(return_value=KRAInstance(False)),
        'ipapython.certdb.unparse_trust_flags':
        Mock(side_effect=my_unparse_trust_flags),
    }

    @patch('ipapython.certdb.NSSDatabase')
    @patch('ipaserver.install.cainstance.CAInstance')
    def test_trust_default_ok(self, mock_ca, mock_certdb):
        """Test what should be the standard case"""
        trust = {
            'ocspSigningCert cert-pki-ca': 'u,u,u',
            'subsystemCert cert-pki-ca': 'u,u,u',
            'auditSigningCert cert-pki-ca': 'u,u,Pu',
            'Server-Cert cert-pki-ca': 'u,u,u'
        }
        mock_certdb.return_value = mock_NSSDatabase('nssdb', trust=trust)
        mock_ca.return_value = CAInstance()

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertNSSTrust(registry)

        self.results = capture_results(f)

        assert len(self.results) == 4

        for result in self.results.results:
            assert result.result == constants.SUCCESS
            assert result.source == 'ipahealthcheck.ipa.certs'
            assert result.check == 'IPACertNSSTrust'
            assert 'cert-pki-ca' in result.kw.get('key')

    @patch('ipapython.certdb.NSSDatabase')
    @patch('ipaserver.install.cainstance.CAInstance')
    def test_trust_ocsp_missing(self, mock_ca, mock_certdb):
        """Test a missing certificate"""
        trust = {
            'subsystemCert cert-pki-ca': 'u,u,u',
            'auditSigningCert cert-pki-ca': 'u,u,Pu',
            'Server-Cert cert-pki-ca': 'u,u,u'
        }

        mock_certdb.return_value = mock_NSSDatabase('nssdb', trust=trust)
        mock_ca.return_value = CAInstance()

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertNSSTrust(registry)

        self.results = capture_results(f)

        # The check reports success for those that it found and are correct and
        # reports missing certs last.
        num = len(self.results.results) - 2
        for r in range(0, num):
            result = self.results.results[r]
            assert result.result == constants.SUCCESS
            assert result.source == 'ipahealthcheck.ipa.certs'
            assert result.check == 'IPACertNSSTrust'
            assert 'cert-pki-ca' in result.kw.get('key')

        result = self.results.results[-1]

        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertNSSTrust'
        assert result.kw.get('key') == 'ocspSigningCert cert-pki-ca'
        assert result.kw.get('nickname') == 'ocspSigningCert cert-pki-ca'
        assert result.kw.get('dbdir') == paths.PKI_TOMCAT_ALIAS_DIR

        assert len(self.results) == 4

    @patch('ipapython.certdb.NSSDatabase')
    @patch('ipaserver.install.cainstance.CAInstance')
    def test_trust_bad(self, mock_ca, mock_certdb):
        """Test multiple unexpected trust flags"""
        trust = {
            'ocspSigningCert cert-pki-ca': 'u,u,u',
            'subsystemCert cert-pki-ca': 'X,u,u',
            'auditSigningCert cert-pki-ca': 'u,u,Pu',
            'Server-Cert cert-pki-ca': 'X,u,u'
        }
        mock_certdb.return_value = mock_NSSDatabase('nssdb', trust=trust)
        mock_ca.return_value = CAInstance()

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertNSSTrust(registry)

        self.results = capture_results(f)

        result = self.results.results[1]

        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertNSSTrust'
        assert result.kw.get('key') == 'subsystemCert cert-pki-ca'
        assert result.kw.get('got') == 'X,u,u'
        assert result.kw.get('expected') == 'u,u,u'

        result = self.results.results[3]

        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertNSSTrust'
        assert result.kw.get('key') == 'Server-Cert cert-pki-ca'
        assert result.kw.get('got') == 'X,u,u'
        assert result.kw.get('expected') == 'u,u,u'

        assert len(self.results) == 4

    @patch('ipaserver.install.cainstance.CAInstance')
    def test_trust_caless(self, mock_cainstance):
        """Nothing to check if the master is CALess"""

        mock_cainstance.return_value = CAInstance(False)

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertNSSTrust(registry)

        self.results = capture_results(f)

        assert len(self.results) == 0

    @patch('ipahealthcheck.ipa.certs.get_token_password')
    @patch('ipapython.certdb.NSSDatabase')
    @patch('ipaserver.install.cainstance.CAInstance')
    def test_trust_token_ok(self, mock_ca, mock_certdb, mock_password):
        """Test what should be the standard token case"""
        trust = {
            'hsm:ocspSigningCert cert-pki-ca': 'u,u,u',
            'hsm:subsystemCert cert-pki-ca': 'u,u,u',
            'hsm:auditSigningCert cert-pki-ca': 'u,u,Pu',
            'Server-Cert cert-pki-ca': 'u,u,u'
        }
        mock_certdb.return_value = mock_NSSDatabase('nssdb', token='hsm',
                                                    trust=trust)
        mock_ca.return_value = CAInstance(enabled=True, hsm_enabled=True,
                                          token='hsm')
        mock_password.return_value = 'Secret123'

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertNSSTrust(registry)

        self.results = capture_results(f)

        assert len(self.results) == 4

        for result in self.results.results:
            assert result.result == constants.SUCCESS
            assert result.source == 'ipahealthcheck.ipa.certs'
            assert result.check == 'IPACertNSSTrust'
            assert 'cert-pki-ca' in result.kw.get('key')

    @patch('ipahealthcheck.ipa.certs.get_token_password')
    @patch('ipapython.certdb.NSSDatabase')
    @patch('ipaserver.install.cainstance.CAInstance')
    def test_trust_token_bad(self, mock_ca, mock_certdb, mock_password):
        """Test multiple unexpected trust flags"""
        trust = {
            'hsm:ocspSigningCert cert-pki-ca': 'u,u,u',
            'hsm:subsystemCert cert-pki-ca': 'X,u,u',
            'hsm:auditSigningCert cert-pki-ca': 'u,u,Pu',
            'Server-Cert cert-pki-ca': 'X,u,u'
        }

        mock_certdb.return_value = mock_NSSDatabase('nssdb', token='hsm',
                                                    trust=trust)
        mock_ca.return_value = CAInstance(enabled=True, hsm_enabled=True,
                                          token='hsm')
        mock_password.return_value = 'Secret123'

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertNSSTrust(registry)

        self.results = capture_results(f)

        result = self.results.results[1]

        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertNSSTrust'
        assert result.kw.get('key') == 'hsm:subsystemCert cert-pki-ca'
        assert result.kw.get('got') == 'X,u,u'
        assert result.kw.get('expected') == 'u,u,u'

        result = self.results.results[3]

        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertNSSTrust'
        assert result.kw.get('key') == 'Server-Cert cert-pki-ca'
        assert result.kw.get('got') == 'X,u,u'
        assert result.kw.get('expected') == 'u,u,u'



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

from ipahealthcheck.core.plugin import Plugin


class ServiceCheck(Plugin):
    def __init__(self, registry):
        super().__init__(registry)
        self.service = None
        self.service_name = None

    def check(self, instance=''):
        raise NotImplementedError



#
# Copyright (C) 2022 FreeIPA Contributors see COPYING for license
#

import argparse
import os
import tempfile
from unittest.mock import patch

from ipahealthcheck.core import constants
from ipahealthcheck.core.core import RunChecks
from ipahealthcheck.core.plugin import Result, Results, Plugin, duration

from ipahealthcheck.system.plugin import Registry

from ipahealthcheck.core.output import output_registry, Output

options = argparse.Namespace(check=None, source=None, debug=False,
                             indent=2, list_sources=False,
                             output_type='suppresstest', output_file=None,
                             verbose=False, version=False, config=None)

outputdata = None


@output_registry
class SuppressTest(Output):
    """Test suppression"""
    options = ()

    def generate(self, data):
        global outputdata

        outputdata = data


class UserPlugin(Plugin):
    pass


class UserRegistry(Registry):
    def initialize(self, framework, config, options=None):
        pass


registry = UserRegistry()


@registry
class PluginOne(UserPlugin):
    @duration
    def check(self):
        yield Result(self, constants.ERROR, key="test1", msg="test1")


@registry
class PluginTwo(UserPlugin):
    @duration
    def check(self):
        yield Result(self, constants.ERROR, key="test2", msg="test2")


@patch('ipahealthcheck.core.core.run_service_plugins')
@patch('ipahealthcheck.core.core.parse_options')
@patch('ipahealthcheck.core.core.find_registries')
def test_suppress_none(mock_find, mock_parse, mock_service):
    """
    Test suppressing plugins
    """
    global outputdata
    mock_service.return_value = (Results(), [])
    mock_parse.return_value = options
    mock_find.return_value = {'test': registry}

    outputdata = None

    fd, config_path = tempfile.mkstemp()
    os.close(fd)
    with open(config_path, "w") as fd:
        fd.write('[default]\n')

    try:
        run = RunChecks(['test'], config_path)

        run.run_healthcheck()
        assert len(outputdata) == 2
    finally:
        os.remove(config_path)


@patch('ipahealthcheck.core.core.run_service_plugins')
@patch('ipahealthcheck.core.core.parse_options')
@patch('ipahealthcheck.core.core.find_registries')
def test_suppress_source(mock_find, mock_parse, mock_service):
    """
    Test suppressing plugins
    """
    global outputdata
    mock_service.return_value = (Results(), [])
    mock_parse.return_value = options
    mock_find.return_value = {'test': registry}

    outputdata = None

    fd, config_path = tempfile.mkstemp()
    os.close(fd)
    with open(config_path, "w") as fd:
        fd.write('[default]\n')
        fd.write('[excludes]\n')
        fd.write('source=test_suppress\n')

    try:
        run = RunChecks(['test'], config_path)

        run.run_healthcheck()
        assert len(outputdata) == 0
    finally:
        os.remove(config_path)


@patch('ipahealthcheck.core.core.run_service_plugins')
@patch('ipahealthcheck.core.core.parse_options')
@patch('ipahealthcheck.core.core.find_registries')
def test_suppress_check(mock_find, mock_parse, mock_service):
    """
    Test suppressing plugins
    """
    global outputdata
    mock_service.return_value = (Results(), [])
    mock_parse.return_value = options
    mock_find.return_value = {'test': registry}

    outputdata = None

    fd, config_path = tempfile.mkstemp()
    os.close(fd)
    with open(config_path, "w") as fd:
        fd.write('[default]\n')
        fd.write('[excludes]\n')
        fd.write('check=PluginOne\n')

    try:
        run = RunChecks(['test'], config_path)

        run.run_healthcheck()

        assert len(outputdata) == 1
        assert outputdata[0].get('check') == 'PluginTwo'
    finally:
        os.remove(config_path)


@patch('ipahealthcheck.core.core.run_service_plugins')
@patch('ipahealthcheck.core.core.parse_options')
@patch('ipahealthcheck.core.core.find_registries')
def test_suppress_key(mock_find, mock_parse, mock_service):
    """
    Test suppressing plugins
    """
    global outputdata
    mock_service.return_value = (Results(), [])
    mock_parse.return_value = options
    mock_find.return_value = {'test': registry}

    outputdata = None

    fd, config_path = tempfile.mkstemp()
    os.close(fd)
    with open(config_path, "w") as fd:
        fd.write('[default]\n')
        fd.write('[excludes]\n')
        fd.write('key=test2\n')

    try:
        run = RunChecks(['test'], config_path)

        run.run_healthcheck()

        assert len(outputdata) == 1
        assert outputdata[0].get('check') == 'PluginOne'
        assert outputdata[0].get('kw').get('msg') == 'test1'
    finally:
        os.remove(config_path)



#
# Copyright (C) 2020 FreeIPA Contributors see COPYING for license
#

from copy import deepcopy
import json
import logging
from os import listdir
from os.path import isfile, join

from ipahealthcheck.core.plugin import Plugin, Registry
from ipalib import api


logger = logging.getLogger()

def find_checks(data, source, check):
    """Look through the dict for a matching source and check.

       data: dict of source and check output
       source: name of source to find
       check: name of check to find

       Returns list of contents of source + check or empty list
    """
    rval = []
    for d in data:
        if d.get('source') == source and d.get('check') == check:
            rval.append(d)

    return rval


def get_masters(data):
    """
    Return the list of known masters

    This is determined from the list of loaded healthcheck logs. It
    is possible that mixed versions are used so some may not be
    reporting the full list of masters, so check them all, and raise
    an exception if the list cannot be determined.
    """
    test_masters = list(data)
    masters = None
    for master in test_masters:
        output = find_checks(data[master], 'ipahealthcheck.ipa.meta',
                             'IPAMetaCheck')
        if len(output) == 0:
            raise ValueError('Unable to determine full list of masters. '
                             'ipahealthcheck.ipa.meta:IPAMetaCheck not '
                             'found.')

        masters = output[0].get('kw').get('masters')
        if masters:
            return masters

    raise ValueError('Unable to determine full list of masters. '
                     'None of ipahealthcheck.ipa.meta:IPAMetaCheck '
                     'contain masters.')


class ClusterPlugin(Plugin):
    pass


class ClusterRegistry(Registry):
    def __init__(self):
        super().__init__()
        self.json = None

    def initialize(self, framework, config, options=None):
        super().initialize(framework, config, options)

        self.json = {}

        self.load_files(options.dir)

        if not api.isdone('finalize'):
            if not api.isdone('bootstrap'):
                api.bootstrap(in_server=True,
                              context='ipahealthcheck',
                              log=None)
            if not api.isdone('finalize'):
                api.finalize()

    def load_files(self, dir):
        if self.json:
            return

        files = [f for f in listdir(dir) if isfile(join(dir, f))]
        for file in files:
            fname = join(dir, file)
            logger.debug("Reading %s", fname)
            try:
                with open(fname, 'r') as fd:
                    data = fd.read()
            except Exception as e:
                logger.error("Unable to read %s: %s", fname, e)
                continue

            try:
                data = json.loads(data)
            except Exception as e:
                logger.error("Unable to parse JSON in %s: %s", fname, e)
                continue

            meta = find_checks(data, 'ipahealthcheck.meta.core',
                                   'MetaCheck')
            if meta:
                fqdn = meta[0].get('kw').get('fqdn')
                self.json[fqdn] = deepcopy(data)
            else:
                logger.error("No fqdn defined in JSON in %s", fname)
                continue


registry = ClusterRegistry()



#
# Copyright (C) 2022 FreeIPA Contributors see COPYING for license
#

import argparse
import io
import logging
import os
import tempfile
from contextlib import redirect_stdout
from unittest.mock import patch

from ipahealthcheck.core.core import RunChecks
from ipahealthcheck.core.plugin import Results

options = argparse.Namespace(check=None, source=None, debug=False,
                             indent=2, list_sources=False,
                             output_type='json', output_file=None,
                             verbose=False, version=False, config=None)


@patch('ipahealthcheck.core.core.run_service_plugins')
@patch('ipahealthcheck.core.core.run_plugins')
@patch('ipahealthcheck.core.core.parse_options')
def test_options_merge(mock_parse, mock_run, mock_service):
    """
    Test merging file-based and CLI options
    """
    mock_service.return_value = (Results(), [])
    mock_run.return_value = Results()
    mock_parse.return_value = options
    fd, config_path = tempfile.mkstemp()
    os.close(fd)
    with open(config_path, "w") as fd:
        fd.write('[default]\n')
        fd.write('output_type=human\n')
        fd.write('indent=5\n')

    try:
        run = RunChecks(['ipahealthcheck.registry'], config_path)

        run.run_healthcheck()

        # verify two valus that have defaults with our overriden values
        assert run.options.output_type == 'human'
        assert run.options.indent == 5
    finally:
        os.remove(config_path)


@patch('ipahealthcheck.core.core.run_service_plugins')
@patch('ipahealthcheck.core.core.run_plugins')
@patch('ipahealthcheck.core.core.parse_options')
def test_cfg_file_debug_option(mock_parse, mock_run, mock_service):
    """
    Test if the debug option is respected in the configuration file

    Related: https://bugzilla.redhat.com/show_bug.cgi?id=2079861
    """
    mock_service.return_value = (Results(), [])
    mock_run.return_value = Results()
    mock_parse.return_value = options
    fd, config_path = tempfile.mkstemp()
    os.close(fd)
    with open(config_path, "w") as fd:
        fd.write('[default]\n')
        fd.write('debug=True\n')

    try:
        run = RunChecks([], config_path)
        run.run_healthcheck()

        assert run.options.debug
    finally:
        os.remove(config_path)


@patch('ipahealthcheck.core.core.run_service_plugins')
@patch('ipahealthcheck.core.core.run_plugins')
@patch('ipahealthcheck.core.core.parse_options')
def test_incorrect_output_type_cfg_file(mock_parse, mock_run, mock_service):
    """
    Test the error message is user-friendly if the incorrect output type is
    provided in cfg file

    Related: https://bugzilla.redhat.com/show_bug.cgi?id=2079698
    """
    mock_service.return_value = (Results(), [])
    mock_run.return_value = Results()
    mock_parse.return_value = options
    fd, config_path = tempfile.mkstemp()
    os.close(fd)
    with open(config_path, "w") as fd:
        fd.write('[default]\n')
        fd.write('output_type=42\n')

    try:
        run = RunChecks([], config_path)

        f = io.StringIO()
        with redirect_stdout(f):
            run.run_healthcheck()

        assert "Unknown output-type" in f.getvalue()

    finally:
        os.remove(config_path)


@patch('ipahealthcheck.core.core.run_service_plugins')
@patch('ipahealthcheck.core.core.run_plugins')
@patch('ipahealthcheck.core.core.parse_options')
def test_incorrect_delimiter_cfg_file(mock_parse, mock_run, mock_service,
                                      caplog):
    """
    Test the error message is user-friendly if the incorrect delimiter is
    used in cfg file

    Related: https://bugzilla.redhat.com/show_bug.cgi?id=2079739
    """
    mock_service.return_value = (Results(), [])
    mock_run.return_value = Results()
    mock_parse.return_value = options
    fd, config_path = tempfile.mkstemp()
    os.close(fd)
    with open(config_path, "w") as fd:
        fd.write('[default]\n')
        fd.write('output_type;human\n')

    try:
        run = RunChecks([], config_path)

        with caplog.at_level(logging.ERROR):
            run.run_healthcheck()

        assert "contains parsing errors" in caplog.text

    finally:
        os.remove(config_path)



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

import copy
from datetime import datetime, timedelta, timezone

from ipaplatform.paths import paths

# Fake certmonger tracked request list. This is similar but can be
# distinct from the value from the overrident get_defaults() method.
template = paths.CERTMONGER_COMMAND_TEMPLATE

CERT_EXPIRATION_DAYS = 30

pristine_cm_requests = [
    {
        'nickname': '1234',
        'cert-file': paths.RA_AGENT_PEM,
        'key-file': paths.RA_AGENT_KEY,
        'ca-name': 'dogtag-ipa-ca-renew-agent',
        'template_profile': 'caSubsystemCert',
        'cert-storage': 'FILE',
        'cert-presave-command': template % 'renew_ra_cert_pre',
        'cert-postsave-command': template % 'renew_ra_cert',
        'not-valid-after': (
            int(
                datetime(1970, 1, 1, 0, 17, 4, tzinfo=timezone.utc).timestamp()
            )
        ),
    },
    {
        'nickname': '5678',
        'cert-file': paths.HTTPD_CERT_FILE,
        'key-file': paths.HTTPD_KEY_FILE,
        'ca-name': 'IPA',
        'template_profile': 'caIPAserviceCert',
        'cert-storage': 'FILE',
        'cert-postsave-command': template % 'restart_httpd',
        'not-valid-after': (
            int(
                (
                    datetime.now(timezone.utc) +
                    timedelta(days=CERT_EXPIRATION_DAYS + 1)
                ).timestamp()
            )
        ),
    },
]


class dbus_results:
    """Class to manage the results returned by dbus"""
    def __init__(self):
        self.requests = copy.deepcopy(pristine_cm_requests)

    def __iter__(self):
        for entry in self.requests:
            yield entry

    def __len__(self):
        return len(self.requests)

    def __getitem__(self, index):
        return self.requests[index]

    def append(self, entry):
        self.requests.append(entry)

    def remove(self, index):
        self.requests.remove(self.requests[index])

    def __repr__(self):
        return repr(self.requests)


cm_requests = []


class mock_property:
    def __init__(self, index):
        self.index = index

    def Get(self, object_path, name):
        """Always return a match"""
        if self.index is None:
            return None
        return cm_requests[self.index].get(name)


class mock_dbus:
    """Create a fake dbus representation of a tracked certificate

       The index is used to look up values within the cm_requests
       list of known tracked certificates.
    """
    def __init__(self, request_id):
        self.index = None
        for i, cm_request in enumerate(cm_requests):
            if request_id == cm_request.get('nickname'):
                self.index = i
                break
        self.prop_if = mock_property(self.index)
        self.obj_if = mock_obj_if(self.index)


class mock_obj_if:
    def __init__(self, index):
        self.index = index

    def find_request_by_nickname(self, nickname):
        return None

    def get_requests(self):
        """Return list of request ids that dbus would have returned"""
        return [n.get('nickname') for n in cm_requests]

    def get_nickname(self):
        """Retrieve the certmonger CA nickname"""
        if self.index is None:
            return None
        return cm_requests[self.index].get('ca-name')

    def get_ca(self):
        """Return the CA name for the current request"""
        return cm_requests[self.index].get('nickname')


class _certmonger:
    """An empty object, not needed directly for testing

       Needed to keep the real certmonger from blowing up.
    """
    def __init__(self):
        self.obj_if = mock_obj_if(None)
        self.bus = None


def create_mock_dbus(bus, parent, object_path, object_dbus_interface,
                     parent_dbus_interface=None, property_interface=False):
    """Create a fake dbus object for a given path (request_id)"""
    return mock_dbus(object_path)


def get_expected_requests():
    """The list of requests known by the IPACertCheck plugin

       The list is copied and the nickname popped off to match the
       format that the check uses.

       nickname has two meanings in certmonger: the request id and
       the NSS nickname.
    """
    requests = copy.deepcopy(pristine_cm_requests)
    for request in requests:
        try:
            request.pop('nickname')
            request.pop('not-valid-after')
        except KeyError:
            pass

    return requests


def set_requests(add=None, remove=None):
    """Set the list of requests within a test"""
    global cm_requests
    cm_requests = dbus_results()
    if add is not None:
        cm_requests.append(add)
    if remove is not None:
        cm_requests.remove(remove)



#
# Copyright (C) 2020 FreeIPA Contributors see COPYING for license
#

from ipahealthcheck.ds.plugin import DSPlugin, registry
from lib389.dseldif import FSChecks


@registry
class FSCheck(DSPlugin):
    """
    Check the FS for permissions issues impacting DS
    """
    check_class = FSChecks



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

import pwd
import posix
from ipahealthcheck.core.files import FileCheck
from ipahealthcheck.core import constants
from ipahealthcheck.core.plugin import Results
from unittest.mock import patch

from util import capture_results

nobody = pwd.getpwnam('nobody')

# Mock files to test
files = (('foo', 'root', 'root', '0660'),
         ('bar', 'nobody', 'nobody', '0664'),
         ('baz', ('root', 'nobody'), ('root', 'nobody'), '0664'),
         ('fiz', ('root', 'bin'), ('root', 'bin'), '0664'),
         ('zap', ('root', 'bin'), ('root', 'bin'), ('0664', '0640'),))


def make_stat(mode=33200, uid=0, gid=0):
    """Return a mocked-up stat.

       The default is:
            mode = 0660
            owner = root
            group = root

       Cheat sheet equivalents:
           0600 = 33152
           0640 = 33184
           0644 = 33188
           0660 = 33200
           0666 = 33206
    """
    # (mode, ino, dev, nlink, uid, gid, size, atime, mtime, ctime)
    return posix.stat_result((mode, 1, 42, 1, uid, gid, 0, 1, 1, 1,))


def get_results(results, type):
    """Pull out the type of results I want to look at: owner, group or mode"""
    my_results = Results()
    for r in results.results:
        kw = r.kw
        if kw.get('type') != type:
            continue
        my_results.add(r)

    return my_results


@patch('os.stat')
def test_files_owner(mock_stat):
    """
    Test the file owner.

    Our mocked files want root, nobody, (root, nobody), (root, root).
    """
    f = FileCheck()
    f.files = files

    mock_stat.return_value = make_stat()
    results = capture_results(f)

    my_results = get_results(results, 'owner')
    assert my_results.results[0].result == constants.SUCCESS
    assert my_results.results[1].result == constants.WARNING
    assert my_results.results[2].result == constants.SUCCESS
    assert my_results.results[3].result == constants.SUCCESS

    mock_stat.return_value = make_stat(uid=nobody.pw_uid)
    results = capture_results(f)
    my_results = get_results(results, 'owner')
    assert my_results.results[0].result == constants.WARNING
    assert my_results.results[0].kw.get('got') == 'nobody'
    assert my_results.results[0].kw.get('expected') == 'root'
    assert my_results.results[0].kw.get('type') == 'owner'

    assert my_results.results[1].result == constants.SUCCESS
    assert my_results.results[2].result == constants.SUCCESS

    assert my_results.results[3].result == constants.WARNING
    assert my_results.results[3].kw.get('got') == 'nobody'
    assert my_results.results[3].kw.get('expected') == 'root,bin'
    assert my_results.results[3].kw.get('type') == 'owner'
    assert my_results.results[3].kw.get('msg') == \
        'Ownership of fiz is nobody and should be one of root,bin'

    assert my_results.results[4].result == constants.WARNING
    assert my_results.results[4].kw.get('got') == 'nobody'
    assert my_results.results[4].kw.get('expected') == 'root,bin'
    assert my_results.results[4].kw.get('type') == 'owner'


@patch('os.stat')
def test_files_group(mock_stat):
    """
    Test the file group.

    Our mocked files want root, nobody, (root, nobody), (root, root).
    """
    f = FileCheck()
    f.files = files

    mock_stat.return_value = make_stat()
    results = capture_results(f)

    my_results = get_results(results, 'group')
    assert my_results.results[0].result == constants.SUCCESS
    assert my_results.results[1].result == constants.WARNING
    assert my_results.results[2].result == constants.SUCCESS
    assert my_results.results[3].result == constants.SUCCESS

    mock_stat.return_value = make_stat(gid=nobody.pw_gid)
    results = capture_results(f)
    my_results = get_results(results, 'group')
    assert my_results.results[0].result == constants.WARNING
    assert my_results.results[0].kw.get('got') == 'nobody'
    assert my_results.results[0].kw.get('expected') == 'root'
    assert my_results.results[0].kw.get('type') == 'group'

    assert my_results.results[1].result == constants.SUCCESS
    assert my_results.results[2].result == constants.SUCCESS

    assert my_results.results[3].result == constants.WARNING
    assert my_results.results[3].kw.get('got') == 'nobody'
    assert my_results.results[3].kw.get('expected') == 'root,bin'
    assert my_results.results[3].kw.get('type') == 'group'
    assert my_results.results[3].kw.get('msg') == \
        'Group of fiz is nobody and should be one of root,bin'

    assert my_results.results[4].result == constants.WARNING
    assert my_results.results[4].kw.get('got') == 'nobody'
    assert my_results.results[4].kw.get('expected') == 'root,bin'
    assert my_results.results[4].kw.get('type') == 'group'


@patch('os.stat')
def test_files_mode(mock_stat):
    mock_stat.return_value = make_stat()

    f = FileCheck()
    f.files = files

    results = capture_results(f)

    my_results = get_results(results, 'mode')
    assert my_results.results[0].result == constants.SUCCESS
    assert my_results.results[1].result == constants.ERROR

    # Too restrictive
    mock_stat.return_value = make_stat(mode=33152)  # 0600
    results = capture_results(f)
    my_results = get_results(results, 'mode')
    assert my_results.results[0].result == constants.ERROR
    assert my_results.results[1].result == constants.ERROR
    assert my_results.results[2].result == constants.ERROR
    assert my_results.results[3].result == constants.ERROR
    assert my_results.results[4].result == constants.ERROR

    # Too permissive
    mock_stat.return_value = make_stat(mode=33206)  # 0666
    results = capture_results(f)
    my_results = get_results(results, 'mode')
    assert my_results.results[0].result == constants.WARNING
    assert my_results.results[1].result == constants.WARNING
    assert my_results.results[2].result == constants.WARNING
    assert my_results.results[3].result == constants.WARNING
    assert my_results.results[4].result == constants.WARNING

    # Too restrictive with allowed multi-mode value
    mock_stat.return_value = make_stat(mode=33184)  # 0640
    results = capture_results(f)
    my_results = get_results(results, 'mode')
    assert my_results.results[0].result == constants.ERROR
    assert my_results.results[1].result == constants.ERROR
    assert my_results.results[2].result == constants.ERROR
    assert my_results.results[3].result == constants.ERROR
    assert my_results.results[4].result == constants.SUCCESS


@patch('os.path.exists')
def test_files_not_found(mock_exists):
    mock_exists.return_value = False

    f = FileCheck()
    f.files = files

    results = capture_results(f)

    for type in ('mode', 'group', 'owner'):
        my_results = get_results(results, type)
        assert len(my_results.results) == len(f.files)
        for result in my_results.results:
            assert result.result == constants.SUCCESS
            assert result.kw.get('msg') == 'File does not exist'



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

from util import capture_results
from base import BaseTest
from ipaplatform.paths import paths
from ipahealthcheck.core import config, constants
from ipahealthcheck.ipa.plugin import registry
from ipahealthcheck.ipa.certs import IPACertmongerExpirationCheck
from ipahealthcheck.ipa.certs import IPACAChainExpirationCheck
from unittest.mock import Mock, patch
from mock_certmonger import create_mock_dbus, _certmonger
from mock_certmonger import (
    get_expected_requests,
    set_requests,
    CERT_EXPIRATION_DAYS,
)

from datetime import datetime, timedelta, timezone


class TestExpiration(BaseTest):
    patches = {
        'ipahealthcheck.ipa.certs.get_expected_requests':
        Mock(return_value=get_expected_requests()),
        'ipalib.install.certmonger._cm_dbus_object':
        Mock(side_effect=create_mock_dbus),
        'ipalib.install.certmonger._certmonger':
        Mock(return_value=_certmonger())
    }

    def test_expiration(self):
        set_requests()

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertmongerExpirationCheck(registry)

        f.config.cert_expiration_days = '7'
        self.results = capture_results(f)

        assert len(self.results) == 2

        result = self.results.results[0]
        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertmongerExpirationCheck'
        assert result.kw.get('key') == '1234'
        assert result.kw.get('expiration_date') == '19700101001704Z'

        result = self.results.results[1]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertmongerExpirationCheck'
        assert result.kw.get('key') == '5678'

    def test_expiration_warning(self):
        warning = datetime.now(timezone.utc) + timedelta(days=20)
        replaceme = {
            'nickname': '7777',
            'cert-file': paths.RA_AGENT_PEM,
            'key-file': paths.RA_AGENT_KEY,
            'ca-name': 'dogtag-ipa-ca-renew-agent',
            'not-valid-after': int(warning.timestamp()),
        }

        set_requests(remove=0, add=replaceme)

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertmongerExpirationCheck(registry)

        f.config.cert_expiration_days = str(CERT_EXPIRATION_DAYS)
        self.results = capture_results(f)

        assert len(self.results) == 2

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertmongerExpirationCheck'
        assert result.kw.get('key') == '5678'

        result = self.results.results[1]
        assert result.result == constants.WARNING
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertmongerExpirationCheck'
        assert result.kw.get('key') == '7777'
        assert result.kw.get('days') == 19


class FakeIPACertificate:
    def __init__(self, cert, backend=None, subject=None, not_after=None):
        self.subj = subject
        self.not_after = not_after

    @property
    def subject(self):
        return self.subj

    @property
    def not_valid_after(self):
        return self.not_after


class TestChainExpiration(BaseTest):
    root_ca = 'CN=Certificate Shack Root CA,O=Certificate Shack Ltd'
    sub_ca = 'CN=Certificate Shack Intermediate CA,O=Certificate Shack Ltd'

    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_still_valid(self, mock_load):
        mock_load.return_value = [
            FakeIPACertificate(
                None,
                subject=self.sub_ca,
                not_after=datetime.now(timezone.utc) + timedelta(days=20)
            ),
            FakeIPACertificate(
                None,
                subject=self.root_ca,
                not_after=datetime.now(timezone.utc) + timedelta(days=20)
            )
        ]
        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACAChainExpirationCheck(registry)

        f.config.cert_expiration_days = '7'
        self.results = capture_results(f)

        assert len(self.results) == 2

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACAChainExpirationCheck'
        assert result.kw.get('key') == self.sub_ca

        result = self.results.results[1]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACAChainExpirationCheck'
        assert result.kw.get('key') == self.root_ca

    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_expiring_soon(self, mock_load):
        mock_load.return_value = [
            FakeIPACertificate(
                None,
                subject=self.sub_ca,
                not_after=datetime.now(timezone.utc) +
                timedelta(days=3, minutes=1)
            ),
            FakeIPACertificate(
                None,
                subject=self.root_ca,
                not_after=datetime.now(timezone.utc) +
                timedelta(days=3, minutes=1)
            )
        ]
        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACAChainExpirationCheck(registry)

        f.config.cert_expiration_days = '7'
        self.results = capture_results(f)

        assert len(self.results) == 2

        result = self.results.results[0]
        assert result.result == constants.WARNING
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACAChainExpirationCheck'
        assert result.kw.get('key') == self.sub_ca
        assert result.kw.get('days') == 3
        assert 'expiring' in result.kw.get('msg')

        result = self.results.results[1]
        assert result.result == constants.WARNING
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACAChainExpirationCheck'
        assert result.kw.get('key') == self.root_ca
        assert result.kw.get('days') == 3
        assert 'expiring' in result.kw.get('msg')

    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_all_expired(self, mock_load):
        mock_load.return_value = [
            FakeIPACertificate(
                None,
                subject=self.sub_ca,
                not_after=datetime.now(timezone.utc) + timedelta(days=-3)
            ),
            FakeIPACertificate(
                None,
                subject=self.root_ca,
                not_after=datetime.now(timezone.utc) + timedelta(days=-3)
            )
        ]
        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACAChainExpirationCheck(registry)

        f.config.cert_expiration_days = '7'
        self.results = capture_results(f)

        assert len(self.results) == 2

        result = self.results.results[0]
        assert result.result == constants.CRITICAL
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACAChainExpirationCheck'
        assert result.kw.get('key') == self.sub_ca
        assert 'expired' in result.kw.get('msg')

        result = self.results.results[1]
        assert result.result == constants.CRITICAL
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACAChainExpirationCheck'
        assert result.kw.get('key') == self.root_ca
        assert 'expired' in result.kw.get('msg')

    @patch('ipalib.x509.load_certificate_list_from_file')
    def test_one_expired(self, mock_load):
        mock_load.return_value = [
            FakeIPACertificate(
                None,
                subject=self.sub_ca,
                not_after=datetime.now(timezone.utc) + timedelta(days=-3)
            ),
            FakeIPACertificate(
                None,
                subject=self.root_ca,
                not_after=datetime.now(timezone.utc) + timedelta(days=20)
            )
        ]
        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACAChainExpirationCheck(registry)

        f.config.cert_expiration_days = '7'
        self.results = capture_results(f)

        assert len(self.results) == 2

        result = self.results.results[0]
        assert result.result == constants.CRITICAL
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACAChainExpirationCheck'
        assert result.kw.get('key') == self.sub_ca
        assert 'expired' in result.kw.get('msg')

        result = self.results.results[1]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACAChainExpirationCheck'
        assert result.kw.get('key') == self.root_ca



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

from util import capture_results
from base import BaseTest
from ipahealthcheck.core import config, constants
from ipahealthcheck.ipa.plugin import registry
from ipahealthcheck.ipa.certs import IPACertfileExpirationCheck
from unittest.mock import Mock, patch
from mock_certmonger import create_mock_dbus, _certmonger
from mock_certmonger import (
    get_expected_requests,
    set_requests,
    CERT_EXPIRATION_DAYS,
)

from datetime import datetime, timedelta


class IPACertificate:
    def __init__(self, not_valid_after, serial_number=1):
        self.subject = 'CN=RA AGENT'
        self.issuer = 'CN=ISSUER'
        self.serial_number = serial_number
        self.not_valid_after = not_valid_after


class TestIPACertificateFile(BaseTest):
    patches = {
        'ipahealthcheck.ipa.certs.get_expected_requests':
        Mock(return_value=get_expected_requests()),
        'ipalib.install.certmonger._cm_dbus_object':
        Mock(side_effect=create_mock_dbus),
        'ipalib.install.certmonger._certmonger':
        Mock(return_value=_certmonger()),
    }

    @patch('ipalib.x509.load_certificate_from_file')
    def test_certfile_expiration(self, mock_load_cert):
        set_requests(remove=1)

        cert = IPACertificate(not_valid_after=datetime.utcnow() +
                              timedelta(days=CERT_EXPIRATION_DAYS))
        mock_load_cert.return_value = cert

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertfileExpirationCheck(registry)

        f.config.cert_expiration_days = '28'
        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.SUCCESS
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertfileExpirationCheck'
        assert result.kw.get('key') == '1234'

    @patch('ipalib.x509.load_certificate_from_file')
    def test_certfile_expiration_warning(self, mock_load_cert):
        set_requests(remove=1)

        cert = IPACertificate(not_valid_after=datetime.utcnow() +
                              timedelta(days=7))
        mock_load_cert.return_value = cert

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertfileExpirationCheck(registry)

        f.config.cert_expiration_days = str(CERT_EXPIRATION_DAYS)
        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.WARNING
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertfileExpirationCheck'
        assert result.kw.get('key') == '1234'
        assert result.kw.get('days') == 6

    @patch('ipalib.x509.load_certificate_from_file')
    def test_certfile_expiration_expired(self, mock_load_cert):
        set_requests(remove=1)

        cert = IPACertificate(not_valid_after=datetime.utcnow() +
                              timedelta(days=-100))
        mock_load_cert.return_value = cert

        framework = object()
        registry.initialize(framework, config.Config)
        f = IPACertfileExpirationCheck(registry)

        f.config.cert_expiration_days = str(CERT_EXPIRATION_DAYS)
        self.results = capture_results(f)

        assert len(self.results) == 1

        result = self.results.results[0]
        assert result.result == constants.ERROR
        assert result.source == 'ipahealthcheck.ipa.certs'
        assert result.check == 'IPACertfileExpirationCheck'
        assert result.kw.get('key') == '1234'
        assert 'expiration_date' in result.kw



#
# Copyright (C) 2019 FreeIPA Contributors see COPYING for license
#

from base import BaseTest
from util import capture_results

from ipahealthcheck.core import config
from ipaclustercheck.ipa.plugin import ClusterRegistry
from ipaclustercheck.ipa.ruv import ClusterRUVCheck

import clusterdata


class RUVRegistry(ClusterRegistry):
    def load_files(self, dir):
        self.json = dir


class Options:
    def __init__(self, data):
        self.data = data

    @property
    def dir(self):
        return self.data


registry = RUVRegistry()


class TestClusterRUV(BaseTest):

    def test_no_ruvs(self):
        """Single master test that has never created a replica

           This type of master will have no RUVs created at all.
        """
        framework = object()
        registry.initialize(framework, config.Config,
                            Options(clusterdata.ONE_MASTER))
        f = ClusterRUVCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 2
        result = self.results.results[0]
        assert result.kw.get('name') == 'dangling_ruv'
        assert result.kw.get('value') == 'No dangling RUVs found'
        result = self.results.results[1]
        assert result.kw.get('name') == 'dangling_csruv'
        assert result.kw.get('value') == 'No dangling CS RUVs found'

    def test_six_ruvs_ok(self):
        """Three master test with each having a CA, no dangling
        """
        framework = object()
        registry.initialize(framework, config.Config,
                            Options(clusterdata.THREE_MASTERS_OK))
        f = ClusterRUVCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 2
        result = self.results.results[0]
        assert result.kw.get('name') == 'dangling_ruv'
        assert result.kw.get('value') == 'No dangling RUVs found'
        result = self.results.results[1]
        assert result.kw.get('name') == 'dangling_csruv'
        assert result.kw.get('value') == 'No dangling CS RUVs found'

    def test_six_ruvs_ipa_bad(self):
        """Three master test with each having a CA, dangling IPA RUV
        """
        framework = object()
        registry.initialize(framework, config.Config,
                            Options(clusterdata.THREE_MASTERS_BAD_IPA_RUV))
        f = ClusterRUVCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 2
        result = self.results.results[0]
        assert result.kw.get('name') == 'dangling_ruv'
        assert result.kw.get('value') == '9'
        result = self.results.results[1]
        assert result.kw.get('name') == 'dangling_csruv'
        assert result.kw.get('value') == 'No dangling CS RUVs found'

    def test_six_ruvs_cs_bad(self):
        """Three master test with each having a CA, dangling CA RUV
        """
        framework = object()
        registry.initialize(framework, config.Config,
                            Options(clusterdata.THREE_MASTERS_BAD_CS_RUV))
        f = ClusterRUVCheck(registry)

        self.results = capture_results(f)

        assert len(self.results) == 2
        result = self.results.results[0]
        assert result.kw.get('name') == 'dangling_ruv'
        assert result.kw.get('value') == 'No dangling RUVs found'
        result = self.results.results[1]
        assert result.kw.get('name') == 'dangling_csruv'
        assert result.kw.get('value') == '9'



#
# Copyright (C) 2020 FreeIPA Contributors see COPYING for license
#

from ipahealthcheck.ds.plugin import DSPlugin, registry
from lib389.config import Config


@registry
class ConfigCheck(DSPlugin):
    """
    Check the DS config for obvious errors
    """
    check_class = Config
